# encoding: utf-8
"""
neighbor.py

Created by Thomas Mangin on 2009-11-05.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from collections import deque

# collections.counter is python2.7 only ..
from exabgp.dep.counter import Counter

from exabgp.protocol.family import AFI

from exabgp.bgp.message import Message
from exabgp.bgp.message.open.holdtime import HoldTime
from exabgp.bgp.message.open.capability import AddPath

from exabgp.reactor.api.encoding import APIOptions

from exabgp.rib import RIB


# The definition of a neighbor (from reading the configuration)
class Neighbor (object):
	def __init__ (self):
		# self.logger should not be used here as long as we do use deepcopy as it contains a Lock
		self.description = ''
		self.router_id = None
		self.local_address = None
		self.peer_address = None
		self.peer_as = None
		self.local_as = None
		self.hold_time = HoldTime(180)
		self.asn4 = None
		self.add_path = 0
		self.md5 = None
		self.ttl = None
		self.group_updates = None
		self.flush = None
		self.adjribout = None

		self.manual_eor = False

		self.api = APIOptions()

		# passive indicate that we do not establish outgoing connections
		self.passive = False
		# the port to listen on ( zero mean that we do not listen )
		self.listen = 0
		# the port to listen on ( zero mean use the environment default )
		self.connect = 0

		# capability
		self.route_refresh = False
		self.graceful_restart = False
		self.multisession = None
		self.add_path = None
		self.aigp = None

		self._families = []
		self.rib = None

		# The routes we have parsed from the configuration
		self.changes = []
		# On signal update, the previous routes so we can compare what changed
		self.backup_changes = []

		self.operational = None
		self.eor = deque()
		self.asm = dict()

		self.messages = deque()
		self.refresh = deque()

		self.counter = Counter()

	def identificator (self):
		# It is possible to :
		# - have multiple exabgp toward one peer on the same host ( use of pid )
		# - have more than once connection toward a peer
		# - each connection has it own neihgbor (hence why identificator is not in Protocol)
		return str(self.peer_address)

	def make_rib (self):
		self.rib = RIB(self.name(),self.adjribout,self._families)

	# will resend all the routes once we reconnect
	def reset_rib (self):
		self.rib.reset()
		self.messages = deque()
		self.refresh = deque()

	# back to square one, all the routes are removed
	def clear_rib (self):
		self.rib.clear()
		self.messages = deque()
		self.refresh = deque()

	def name (self):
		if self.multisession:
			session = '/'.join("%s-%s" % (afi.name(),safi.name()) for (afi,safi) in self.families())
		else:
			session = 'in-open'
		return "neighbor %s local-ip %s local-as %s peer-as %s router-id %s family-allowed %s" % (self.peer_address,self.local_address,self.local_as,self.peer_as,self.router_id,session)

	def families (self):
		# this list() is important .. as we use the function to modify self._families
		return list(self._families)

	def add_family (self, family):
		# the families MUST be sorted for neighbor indexing name to be predictable for API users
		if family not in self.families():
			afi,safi = family
			d = dict()
			d[afi] = [safi,]
			for afi,safi in self._families:
				d.setdefault(afi,[]).append(safi)
			self._families = [(afi,safi) for afi in sorted(d) for safi in sorted(d[afi])]

	def remove_family (self, family):
		if family in self.families():
			self._families.remove(family)

	def missing (self):
		if self.local_address is None:
			return 'local-address'
		if self.peer_address is None:
			return 'peer-address'
		if self.local_as is None:
			return 'local-as'
		if self.peer_as is None:
			return 'peer-as'
		if self.peer_address.afi == AFI.ipv6 and not self.router_id:
			return 'router-id'
		return ''

	# This function only compares the neighbor BUT NOT ITS ROUTES
	def __eq__ (self, other):
		return \
			self.router_id == other.router_id and \
			self.local_address == other.local_address and \
			self.local_as == other.local_as and \
			self.peer_address == other.peer_address and \
			self.peer_as == other.peer_as and \
			self.passive == other.passive and \
			self.listen == other.listen and \
			self.connect == other.connect and \
			self.hold_time == other.hold_time and \
			self.md5 == other.md5 and \
			self.ttl == other.ttl and \
			self.route_refresh == other.route_refresh and \
			self.graceful_restart == other.graceful_restart and \
			self.multisession == other.multisession and \
			self.add_path == other.add_path and \
			self.operational == other.operational and \
			self.group_updates == other.group_updates and \
			self.flush == other.flush and \
			self.adjribout == other.adjribout and \
			self.families() == other.families()

	def __ne__ (self, other):
		return not self.__eq__(other)

	def pprint (self, with_changes=True):
		changes = ''
		if with_changes:
			changes += '\nstatic { '
			for changes in self.rib.incoming.queued_changes():
				changes += '\n\t\t%s' % changes.extensive()
			changes += '\n}'

		families = ''
		for afi,safi in self.families():
			families += '\n\t\t%s %s;' % (afi.name(),safi.name())

		_extension_receive = {
			'receive-parsed':           'parsed',
			'receive-packets':          'packets',
			'receive-parsed':           'parsed',
			'consolidate':              'consolidate',
			'neighbor-changes':         'neighbor-changes',
			Message.CODE.NOTIFICATION:  'notification',
			Message.CODE.OPEN:          'open',
			Message.CODE.KEEPALIVE:     'keepalive',
			Message.CODE.UPDATE:        'update',
			Message.CODE.ROUTE_REFRESH: 'refresh',
			Message.CODE.OPERATIONAL:   'operational',
		}

		_extension_send = {
			'send-packets': 'packets',
		}

		_receive = []
		for api,name in _extension_receive.items():
			_receive.extend(['\t\t\t%s;\n' % name,] if self.api[api] else [])
		receive = ''.join(_receive)

		_send = []
		for api,name in _extension_send.items():
			_send.extend(['\t\t\t%s;\n' % name,] if self.api[api] else [])
		send = ''.join(_send)

		returned = \
			'neighbor %s {\n' \
			'\tdescription "%s";\n' \
			'\trouter-id %s;\n' \
			'\tlocal-address %s;\n' \
			'\tlocal-as %s;\n' \
			'\tpeer-as %s;\n' \
			'\thold-time %s;\n' \
			'\tmanual-eor %s;\n' \
			'%s%s%s%s%s%s%s%s\n' \
			'\tcapability {\n' \
			'%s%s%s%s%s%s%s\t}\n' \
			'\tfamily {%s\n' \
			'\t}\n' \
			'\tprocess {\n' \
			'%s%s\t}%s\n' \
			'}' % (
				self.peer_address,
				self.description,
				self.router_id,
				self.local_address,
				self.local_as,
				self.peer_as,
				self.hold_time,
				'true' if self.manual_eor else 'false',
				'\n\tpassive;\n' if self.passive else '',
				'\n\tlisten %d;\n' % self.listen if self.listen else '',
				'\n\tconnect %d;\n' % self.connect if self.connect else '',
				'\tgroup-updates: %s;\n' % (self.group_updates if self.group_updates else ''),
				'\tauto-flush: %s;\n' % ('true' if self.flush else 'false'),
				'\tadj-rib-out: %s;\n' % ('true' if self.adjribout else 'false'),
				'\tmd5 "%s";\n' % self.md5 if self.md5 else '',
				'\tttl-security: %s;\n' % (self.ttl if self.ttl else ''),
				'\t\tasn4 %s;\n' % ('enable' if self.asn4 else 'disable'),
				'\t\troute-refresh %s;\n' % ('enable' if self.route_refresh else 'disable'),
				'\t\tgraceful-restart %s;\n' % (self.graceful_restart if self.graceful_restart else 'disable'),
				'\t\tadd-path %s;\n' % (AddPath.string[self.add_path] if self.add_path else 'disable'),
				'\t\tmulti-session %s;\n' % ('enable' if self.multisession else 'disable'),
				'\t\toperational %s;\n' % ('enable' if self.operational else 'disable'),
				'\t\taigp %s;\n' % ('enable' if self.aigp else 'disable'),
				families,
				'\t\treceive {\n%s\t\t}\n' % receive if receive else '',
				'\t\tsend {\n%s\t\t}\n' % send if send else '',
				changes
			)
		return returned.replace('\t','  ')

	def __str__ (self):
		return self.pprint(False)



# encoding: utf-8
"""
unknown.py

Created by Thomas Mangin on 2014-06-30.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from exabgp.bgp.message.open.capability.capability import Capability

# ============================================================ UnknownCapability
#


class UnknownCapability (Capability):
	def set (self, capability, data=''):
		self.capability = capability
		self.data = data
		return self

	def __str__ (self):
		if self.capability in Capability.CODE.reserved:
			return 'Reserved %s' % str(self.capability)
		if self.capability in Capability.CODE.unassigned:
			return 'Unassigned %s' % str(self.capability)
		return 'Unknown %s' % str(self.capability)

	def json (self):
		if self.capability in Capability.CODE.reserved:
			iana = 'reserved'
		elif self.capability in Capability.CODE.unassigned:
			iana = 'unassigned'
		else:
			iana = 'unknown'
		return '{ "name": "unknown", "iana": "%s", "value": %d, "raw": "%s" }' % (iana,self.capability,self.data)

	def extract (self):
		return []

	@staticmethod
	def unpack_capability (instance, data, capability):
		return instance.set(capability,data)



# http://teethgrinder.co.uk/perm.php?a=Python-memory-leak-detector
import gc
import inspect


def dump():
	# force collection
	print "\nCollecting GARBAGE:"
	gc.collect()
	# prove they have been collected
	print "\nCollecting GARBAGE:"
	gc.collect()

	print "\nGARBAGE OBJECTS:"
	for x in gc.garbage:
		s = str(x)
		if len(s) > 80:
			s = "%s..." % s[:80]

		print "::", s
		print "		type:", type(x)
		print "   referrers:", len(gc.get_referrers(x))
		try:
			print "	is class:", inspect.isclass(type(x))
			print "	  module:", inspect.getmodule(x)

			lines, line_num = inspect.getsourcelines(type(x))
			print "	line num:", line_num
			for l in lines:
				print "		line:", l.rstrip("\n")
		except Exception:
			pass

		print



# encoding: utf-8
"""
attributes.py

Created by Thomas Mangin on 2009-11-05.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from struct import pack
from struct import unpack

from exabgp.bgp.message.update.attribute.attribute import Attribute


# ========================================================= Local Preference (5)
#

class LocalPreference (Attribute):
	ID = Attribute.CODE.LOCAL_PREF
	FLAG = Attribute.Flag.TRANSITIVE
	CACHING = True

	__slots__ = ['localpref','packed']

	def __init__ (self, localpref, packed=None):
		self.localpref = localpref
		self.packed = self._attribute(packed if packed is not None else pack('!L',localpref))

	def pack (self, negotiated=None):
		return self.packed

	def __len__ (self):
		return 4

	def __str__ (self):
		return str(self.localpref)

	def __cmp__ (self, other):
		if not isinstance(other,self.__class__):
			return -1
		if self.localpref != other.localpref:
			return -1
		return 0

	@classmethod
	def unpack (cls, data, negotiated):
		return cls(unpack('!L',data)[0],data)



# encoding: utf-8
"""
asn4.py

Created by Thomas Mangin on 2014-06-30.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from exabgp.bgp.message.open.asn import ASN
from exabgp.bgp.message.open.capability.capability import Capability

# ========================================================================= ASN4
#


class ASN4 (Capability,ASN):
	ID = Capability.CODE.FOUR_BYTES_ASN

	# This makes python2.6 complain !
	# def __init__ (self, value=0):
	# 	ASN.__init__(self,value)

	def __str__ (self):
		return 'ASN4(%d)' % int(self)

	@staticmethod
	def unpack_capability (instance, data, capability=None):  # pylint: disable=W0613
		# XXX: FIXME: if instance is not ASN(0) we have two ASN - raise
		instance = ASN.unpack(data,ASN4)
		return instance

	def json (self):
		return '{ "name": "asn4", "asn4": %d }' % int(self)



# encoding: utf-8
"""
check.py

Created by Thomas Mangin on 2013-03-18.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""


class TYPE (object):
	NULL     = 0x01
	BOOLEAN  = 0x02
	INTEGER  = 0x04
	STRING   = 0x08
	ARRAY    = 0x10
	HASH     = 0x20


class PRESENCE (object):
	OPTIONAL  = 0x01
	MANDATORY = 0x02


# TYPE CHECK

def null (data):
	return type(data) == type(None)  # noqa


def boolean (data):
	return type(data) == type(True)  # noqa


def integer (data):
	return type(data) == type(0)  # noqa


def string (data):
	return type(data) == type(u'') or type(data) == type('')  # noqa


def array (data):
	return type(data) == type([])  # noqa


def hashtable (data):
	return type(data) == type({})  # noqa


# XXX: Not very good to redefine the keyword object, but this class uses no OO ...

CHECK_TYPE = {
	TYPE.NULL: null,
	TYPE.BOOLEAN: boolean,
	TYPE.INTEGER: integer,
	TYPE.STRING: string,
	TYPE.ARRAY: array,
	TYPE.HASH: hashtable,
}


def kind (kind, data):
	for t in CHECK_TYPE:
		if kind & t:
			if CHECK_TYPE[t](data):
				return True
	return False

# DATA CHECK


def nop (data):
	return True


def uint8 (data):
	return 0 <= data < pow(2,8)


def uint16 (data):
	return 0 <= data < pow(2,16)


def uint32 (data):
	return 0 <= data < pow(2,32)


def float (data):
	return 0 <= data < 3.4 * pow(10,38)  # approximation of max from wikipedia


def ip (data):
	return ipv4(data) or ipv6(data)


def ipv4 (data):  # XXX: improve
	return string(data) and data.count('.') == 3


def ipv6 (data):  # XXX: improve
	return string(data) and ':' in data


def range4 (data):
	return 0 < data <= 32


def range6 (data):
	return 0 < data <= 128


def ipv4_range (data):
	if not data.count('/') == 1:
		return False
	ip,r = data.split('/')
	if not ipv4(ip):
		return False
	if not r.isdigit():
		return False
	if not range4(int(r)):
		return False
	return True


def port (data):
	return 0 <= data < pow(2,16)


def asn16 (data):
	return 1 <= data < pow(2,16)


def asn32 (data):
	return 1 <= data < pow(2,32)
asn = asn32


def md5 (data):
	return len(data) <= 18


def localpreference (data):
	return uint32(data)


def med (data):
	return uint32(data)


def aigp (data):
	return uint32(data)


def originator (data):
	return ipv4(data)


def distinguisher (data):
	parts = data.split(':')
	if len(parts) != 2:
		return False
	_,__ = parts
	return (_.isdigit() and asn16(int(_)) and ipv4(__)) or (ipv4(_) and __.isdigit() and asn16(int(__)))


def pathinformation (data):
	if integer(data):
		return uint32(data)
	if string(data):
		return ipv4(data)
	return False


def watchdog (data):
	return ' ' not in data  # TODO: improve


def split (data):
	return range6(data)

# LIST DATA CHECK
# Those function need to perform type checks before using the data


def aspath (data):
	return integer(data) and data < pow(2,32)


def assequence (data):
	return integer(data) and data < pow(2,32)


def community (data):
	if integer(data):
		return uint32(data)
	if string(data) and data.lower() in ('no-export', 'no-advertise', 'no-export-subconfed', 'nopeer', 'no-peer', 'blackhole'):
		return True
	return array(data) and len(data) == 2 and \
		integer(data[0]) and integer(data[1]) and \
		asn16(data[0]) and uint16(data[1])


def extendedcommunity (data):  # TODO: improve, incomplete see http://tools.ietf.org/rfc/rfc4360.txt
	if integer(data):
		return True
	if string(data) and data.count(':') == 2:
		_,__,___ = data.split(':')
		if _.lower() not in ('origin','target'):
			return False
		return (__.isdigit() and asn16(__) and ipv4(___)) or (ipv4(__) and ___.isdigit() and asn16(___))
	return False


def label (data):
	return integer(data) and 0 <= data < pow(2, 20)  # XXX: SHOULD be taken from Label class


def clusterlist (data):
	return integer(data) and uint8(data)


def aggregator (data):
	if not array(data):
		return False
	if len(data) == 0:
		return True
	if len(data) == 2:
		return \
			integer(data[0]) and string(data[1]) and \
			asn(data[0]) and ipv4(data[1])
	return False


def dscp (data):
	return integer(data) and uint8(data)


# FLOW DATA CHECK
#


def flow_ipv4_range (data):
	if array(data):
		for r in data:
			if not ipv4_range(r):
				return False
	if string(data):
		return ipv4_range(data)
	return False


def _flow_numeric (data, check):
	if not array(data):
		return False
	for et in data:
		if not (array(et) and len(et) == 2 and et[0] in ('>', '<', '=','>=', '<=') and integer(et[1]) and check(et[1])):
			return False
	return True


def flow_port (data):
	return _flow_numeric(data,port)


def _length (data):
	return uint16(data)


def flow_length (data):
	return _flow_numeric(data,_length)


def redirect (data):  # TODO: check that we are not too restrictive with our asn() calls
	parts = data.split(':')
	if len(parts) != 2:
		return False
	_,__ = parts
	if not __.isdigit() and asn16(int(__)):
		return False
	return ipv4(_) or (_.isdigit() and asn16(int(_)))



# encoding: utf-8
"""
community.py

Created by Thomas Mangin on 2009-11-05.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

# ============================================================== Communities (8)
# http://www.iana.org/assignments/bgp-extended-communities

from exabgp.bgp.message.update.attribute.attribute import Attribute
from exabgp.bgp.message.update.attribute.community.community import Community

from exabgp.bgp.message.notification import Notify


class Communities (Attribute):
	ID = Attribute.CODE.COMMUNITY
	FLAG = Attribute.Flag.TRANSITIVE | Attribute.Flag.OPTIONAL

	# __slots__ = ['communities']

	def __init__ (self, communities=None):
		# Must be None as = param is only evaluated once
		if communities:
			self.communities = communities
		else:
			self.communities = []

	def add (self, data):
		return self.communities.append(data)

	def pack (self, negotiated=None):
		if len(self.communities):
			return self._attribute(''.join([c.pack() for c in self.communities]))
		return ''

	def __str__ (self):
		l = len(self.communities)
		if l > 1:
			return "[ %s ]" % " ".join(str(community) for community in self.communities)
		if l == 1:
			return str(self.communities[0])
		return ""

	def json (self):
		return "[ %s ]" % ", ".join(community.json() for community in self.communities)

	@staticmethod
	def unpack (data, negotiated):
		communities = Communities()
		while data:
			if data and len(data) < 4:
				raise Notify(3,1,'could not decode community %s' % str([hex(ord(_)) for _ in data]))
			communities.add(Community.unpack(data[:4],negotiated))
			data = data[4:]
		return communities



#!/usr/bin/env python

"""Healthchecker for exabgp.

This program is to be used as a process for exabgp. It will announce
some VIP depending on the state of a check whose a third-party program
wrapped by this program.

To use, declare this program as a process in your
:file:`/etc/exabgp/exabgp.conf`::

    neighbor 192.0.2.1 {
       router-id 192.0.2.2;
       local-as 64496;
       peer-as 64497;
    }
    process watch-haproxy {
       run python -m exabgp healthcheck --cmd "curl -sf http://127.0.0.1/healthcheck" --label haproxy;
    }
    process watch-mysql {
       run python -m exabgp healthcheck --cmd "mysql -u check -e 'SELECT 1'" --label mysql;
    }

Use :option:`--help` to get options accepted by this program. A
configuration file is also possible. Such a configuration file looks
like this::

     debug
     name = haproxy
     interval = 10
     fast-interval = 1
     command = curl -sf http://127.0.0.1/healthcheck

The left-part of each line is the corresponding long option.

When using label for loopback selection, the provided value should
match the beginning of the label without the interface prefix. In the
example above, this means that you should have addresses on lo
labelled ``lo:haproxy1``, ``lo:haproxy2``, etc.

"""

from __future__ import print_function
from __future__ import unicode_literals

import string
import sys
import os
import subprocess
import re
import logging
import logging.handlers
import argparse
import signal
import time
import collections
try:
    # Python 3.3+ or backport
    from ipaddress import ip_address as _ip_address  # pylint: disable=F0401

    def ip_address(x):
        try:
            x = x.decode('ascii')
        except AttributeError:
            pass
        return _ip_address(x)
except ImportError:
    # Python 2.6, 2.7, 3.2
    from ipaddr import IPAddress as ip_address

logger = logging.getLogger("healthcheck")


def enum (*sequential):
    """Create a simple enumeration."""
    return type(str("Enum"), (), dict(zip(sequential, sequential)))


def parse ():
    """Parse arguments"""
    parser = argparse.ArgumentParser(description=sys.modules[__name__].__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)

    g = parser.add_mutually_exclusive_group()
    g.add_argument("--debug", "-d", action="store_true",
                   default=False,
                   help="enable debugging")
    g.add_argument("--silent", "-s", action="store_true",
                   default=False,
                   help="don't log to console")
    g.add_argument("--syslog-facility", "-sF", metavar="FACILITY",
                   nargs='?',
                   const="daemon",
                   default="daemon",
                   help="log to syslog using FACILITY, default FACILITY is daemon")
    g.add_argument("--no-syslog", action="store_true",
                   help="disable syslog logging")
    parser.add_argument("--name", "-n", metavar="NAME",
                        help="name for this healthchecker")
    parser.add_argument("--config", "-F", metavar="FILE", type=open,
                        help="read configuration from a file")
    parser.add_argument("--pid", "-p", metavar="FILE", type=argparse.FileType('w'),
                        help="write PID to the provided file")
    parser.add_argument("--sudo", action="store_true", default=False,
                        help="use sudo to setup ip addresses")

    g = parser.add_argument_group("checking healthiness")
    g.add_argument("--interval", "-i", metavar='N',
                   default=5,
                   type=float,
                   help="wait N seconds between each healthcheck")
    g.add_argument("--fast-interval", "-f", metavar='N',
                   default=1,
                   type=float, dest="fast",
                   help="when a state change is about to occur, wait N seconds between each healthcheck")
    g.add_argument("--timeout", "-t", metavar='N',
                   default=5,
                   type=int,
                   help="wait N seconds for the check command to execute")
    g.add_argument("--rise", metavar='N',
                   default=3,
                   type=int,
                   help="check N times before considering the service up")
    g.add_argument("--fall", metavar='N',
                   default=3,
                   type=int,
                   help="check N times before considering the service down")
    g.add_argument("--disable", metavar='FILE',
                   type=str,
                   help="if FILE exists, the service is considered disabled")
    g.add_argument("--command", "--cmd", "-c", metavar='CMD',
                   type=str,
                   help="command to use for healthcheck")

    g = parser.add_argument_group("advertising options")
    g.add_argument("--next-hop", "-N", metavar='IP',
                   type=ip_address,
                   help="self IP address to use as next hop")
    g.add_argument("--ip", metavar='IP',
                   type=ip_address, dest="ips", action="append",
                   help="advertise this IP address")
    g.add_argument("--no-ip-setup",
                   action="store_false", dest="ip_setup",
                   help="don't setup missing IP addresses")
    g.add_argument("--dynamic-ip-setup", default=False,
                   action="store_true", dest="ip_dynamic",
                   help="delete existing loopback ips on state down and "\
                        "disabled, then restore loopback when up")
    g.add_argument("--label", default=None,
                   help="use the provided label to match loopback addresses")
    g.add_argument("--start-ip", metavar='N',
                   type=int, default=0,
                   help="index of the first IP in the list of IP addresses")
    g.add_argument("--up-metric", metavar='M',
                   type=int, default=100,
                   help="first IP get the metric M when the service is up")
    g.add_argument("--down-metric", metavar='M',
                   type=int, default=1000,
                   help="first IP get the metric M when the service is down")
    g.add_argument("--disabled-metric", metavar='M',
                   type=int, default=500,
                   help="first IP get the metric M when the service is disabled")
    g.add_argument("--increase", metavar='M',
                   type=int, default=1,
                   help="for each additional IP address increase metric value by W")
    g.add_argument("--community", metavar="COMMUNITY",
                   type=str, default=None,
                   help="announce IPs with the supplied community")
    g.add_argument("--as-path", metavar="ASPATH",
                   type=str, default=None,
                   help="announce IPs with the supplied as-path")
    g.add_argument("--withdraw-on-down", action="store_true",
                   help="Instead of increasing the metric on health failure, withdraw the route")

    g = parser.add_argument_group("reporting")
    g.add_argument("--execute", metavar='CMD',
                   type=str, action="append",
                   help="execute CMD on state change")
    g.add_argument("--up-execute", metavar='CMD',
                   type=str, action="append",
                   help="execute CMD when the service becomes available")
    g.add_argument("--down-execute", metavar='CMD',
                   type=str, action="append",
                   help="execute CMD when the service becomes unavailable")
    g.add_argument("--disabled-execute", metavar='CMD',
                   type=str, action="append",
                   help="execute CMD when the service is disabled")

    options = parser.parse_args()
    if options.config is not None:
        # A configuration file has been provided. Read each line and
        # build an equivalent command line.
        args = sum(["--{0}".format(l.strip()).split("=", 1)
                    for l in options.config.readlines()
                    if not l.strip().startswith("#") and l.strip()], [])
        args = [x.strip() for x in args]
        args.extend(sys.argv[1:])
        options = parser.parse_args(args)
    return options


def setup_logging (debug, silent, name, syslog_facility, syslog):
    """Setup logger"""

    def syslog_address():
        """Return a sensible syslog address"""
        if sys.platform == "darwin":
            return "/var/run/syslog"
        if sys.platform.startswith("freebsd"):
            return "/var/run/log"
        if sys.platform.startswith("netbsd"):
            return "/var/run/log"
        if sys.platform.startswith("linux"):
            return "/dev/log"
        raise EnvironmentError("Unable to guess syslog address for your "
                               "platform, try to disable syslog")

    logger.setLevel(debug and logging.DEBUG or logging.INFO)
    enable_syslog = syslog and not debug
    # To syslog
    if enable_syslog:
        facility = getattr(logging.handlers.SysLogHandler,
                           "LOG_{0}".format(string.upper(syslog_facility)))
        sh = logging.handlers.SysLogHandler(address=str(syslog_address()),
                                            facility=facility)
        if name:
            healthcheck_name = "healthcheck-{0}".format(name)
        else:
            healthcheck_name = "healthcheck"
        sh.setFormatter(logging.Formatter(
            "{0}[{1}]: %(message)s".format(
                healthcheck_name,
                os.getpid())))
        logger.addHandler(sh)
    # To console
    if sys.stderr.isatty() and not silent:
        ch = logging.StreamHandler()
        ch.setFormatter(logging.Formatter(
            "%(levelname)s[%(name)s] %(message)s"))
        logger.addHandler(ch)


def loopback_ips (label):
    """Retrieve loopback IP addresses"""
    logger.debug("Retrieve loopback IP addresses")
    addresses = []

    if sys.platform.startswith("linux"):
        # Use "ip" (ifconfig is not able to see all addresses)
        ipre = re.compile(r"^(?P<index>\d+):\s+(?P<name>\S+)\s+inet6?\s+(?P<ip>[\da-f.:]+)/(?P<netmask>\d+)\s+.*")
        labelre = re.compile(r".*\s+lo:(?P<label>\S+)\s+.*")
        cmd = subprocess.Popen("/sbin/ip -o address show dev lo".split(), shell=False, stdout=subprocess.PIPE)
    else:
        # Try with ifconfig
        ipre = re.compile(r"^inet6?\s+(alias\s+)?(?P<ip>[\da-f.:]+)\s+(?:netmask 0x(?P<netmask>[0-9a-f]+)|prefixlen (?P<mask>\d+)).*")
        cmd = subprocess.Popen("/sbin/ifconfig lo0".split(), shell=False, stdout=subprocess.PIPE)
        labelre = re.compile(r"")
    for line in cmd.stdout:
        line = line.decode("ascii", "ignore").strip()
        mo = ipre.match(line)
        if not mo:
            continue
        ip = ip_address(mo.group("ip"))
        if not ip.is_loopback:
            if label:
                lmo = labelre.match(line)
                if not lmo or not lmo.group("label").startswith(label):
                    continue
            addresses.append(ip)
    logger.debug("Loopback addresses: {0}".format(addresses))
    return addresses


def setup_ips (ips, label, sudo=False):
    """Setup missing IP on loopback interface"""
    existing = set(loopback_ips(label))
    toadd = set(ips) - existing
    for ip in toadd:
        logger.debug("Setup loopback IP address {0}".format(ip))
        with open(os.devnull, "w") as fnull:
            cmd = ["ip", "address", "add", str(ip), "dev", "lo"]
            if sudo:
                cmd.insert(0, "sudo")
            if label:
                cmd += ["label", "lo:{0}".format(label)]
            subprocess.check_call(
                cmd, stdout=fnull, stderr=fnull)

def remove_ips (ips, label, sudo=False):
    """Remove existing IP on loopback interface"""
    existing = set(loopback_ips(label))

    # Get intersection of IPs (ips setup, and IPs configured by ExaBGP)
    toremove = existing.intersection(ips)
    for ip in toremove:
        logger.debug("Remove loopback IP address {0}".format(ip))
        with open(os.devnull, "w") as fnull:
            cmd = ["ip", "address", "delete", str(ip), "dev", "lo"]
            if sudo:
                cmd.insert(0, "sudo")
            if label:
                cmd += ["label", "lo:{0}".format(label)]
            subprocess.check_call(
                cmd, stdout=fnull, stderr=fnull)


def setpgrp_preexec_fn ():
    os.setpgrp()


def check (cmd, timeout):
    """Check the return code of the given command.

    :param cmd: command to execute. If :keyword:`None`, no command is executed.
    :param timeout: how much time we should wait for command completion.
    :return: :keyword:`True` if the command was successful or
             :keyword:`False` if not or if the timeout was triggered.
    """
    if cmd is None:
        return True

    class Alarm(Exception):
        pass

    def alarm_handler (number, frame):  # pylint: disable=W0613
        raise Alarm()

    logger.debug("Checking command {0}".format(repr(cmd)))
    p = subprocess.Popen(cmd, shell=True,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.STDOUT,
                         preexec_fn=setpgrp_preexec_fn)
    if timeout:
        signal.signal(signal.SIGALRM, alarm_handler)
        signal.alarm(timeout)
    try:
        stdout = None
        stdout, _ = p.communicate()
        if timeout:
            signal.alarm(0)
        if p.returncode != 0:
            logger.warn("Check command was unsuccessful: {0}".format(p.returncode))
            if stdout.strip():
                logger.info("Output of check command: {0}".format(stdout))
            return False
        logger.debug("Command was executed successfully {0} {1}".format(p.returncode, stdout))
        return True
    except Alarm:
        logger.warn("Timeout ({0}) while running check command {1}".format(timeout, cmd))
        os.killpg(p.pid, signal.SIGKILL)
        return False


def loop (options):
    """Main loop."""
    states = enum(
        "INIT",                 # Initial state
        "DISABLED",             # Disabled state
        "RISING",               # Checks are currently succeeding.
        "FALLING",              # Checks are currently failing.
        "UP",                   # Service is considered as up.
        "DOWN",                 # Service is considered as down.
    )
    state = states.INIT

    def exabgp (target):
        """Communicate new state to ExaBGP"""
        if target not in (states.UP, states.DOWN, states.DISABLED):
            return
        if target in (states.DOWN, states.DISABLED) and options.ip_dynamic:
            logger.info("service down, deleting loopback ips")
            remove_ips(options.ips, options.label, options.sudo)
        if target == states.UP and options.ip_dynamic:
            logger.info("service up, restoring loopback ips")
            setup_ips(options.ips, options.label, options.sudo)

        logger.info("send announces for {0} state to ExaBGP".format(target))
        metric = vars(options).get("{0}_metric".format(str(target).lower()))
        for ip in options.ips:
            if options.withdraw_on_down:
                command = "announce" if target is states.UP else "withdraw"
            else:
                command = "announce"
            announce = "route {0}/{1} next-hop {2}".format(str(ip),
                                                           ip.max_prefixlen,
                                                           options.next_hop or "self")
            if command == "announce":
                announce = "{0} med {1}".format(announce, metric)
                if options.community:
                    announce = "{0} community [ {1} ]".format(announce,
                                                              options.community)
                if options.as_path:
                    announce = "{0} as-path [ {1} ]".format(announce,
                                                              options.as_path)
            logger.debug("exabgp: {0} {1}".format(command, announce))
            print("{0} {1}".format(command, announce))
            metric += options.increase
        sys.stdout.flush()

    def trigger (target):
        """Trigger a state change and execute the appropriate commands"""
        # Shortcut for RISING->UP and FALLING->UP
        if target == states.RISING and options.rise <= 1:
            target = states.UP
        elif target == states.FALLING and options.fall <= 1:
            target = states.DOWN

        # Log and execute commands
        logger.debug("Transition to {0}".format(str(target)))
        cmds = []
        cmds.extend(vars(options).get("{0}_execute".format(str(target).lower()), []) or [])
        cmds.extend(vars(options).get("execute", []) or [])
        for cmd in cmds:
            logger.debug("Transition to {0}, execute `{1}`".format(str(target), cmd))
            env = os.environ.copy()
            env.update({"STATE": str(target)})
            with open(os.devnull, "w") as fnull:
                subprocess.call(
                    cmd, shell=True, stdout=fnull, stderr=fnull, env=env)

        return target

    checks = 0
    while True:
        disabled = options.disable is not None and os.path.exists(options.disable)
        successful = disabled or check(options.command, options.timeout)
        # FSM
        if state != states.DISABLED and disabled:
            state = trigger(states.DISABLED)
        elif state == states.INIT:
            if successful and options.rise <= 1:
                state = trigger(states.UP)
            elif successful:
                state = trigger(states.RISING)
                checks = 1
            else:
                state = trigger(states.FALLING)
                checks = 1
        elif state == states.DISABLED:
            if not disabled:
                state = trigger(states.INIT)
        elif state == states.RISING:
            if successful:
                checks += 1
                if checks >= options.rise:
                    state = trigger(states.UP)
            else:
                state = trigger(states.FALLING)
                checks = 1
        elif state == states.FALLING:
            if not successful:
                checks += 1
                if checks >= options.fall:
                    state = trigger(states.DOWN)
            else:
                state = trigger(states.RISING)
                checks = 1
        elif state == states.UP:
            if not successful:
                state = trigger(states.FALLING)
                checks = 1
        elif state == states.DOWN:
            if successful:
                state = trigger(states.RISING)
                checks = 1
        else:
            raise ValueError("Unhandled state: {0}".format(str(state)))

        # Send announces. We announce them on a regular basis in case
        # we lose connection with a peer.
        exabgp(state)

        # How much we should sleep?
        if state in (states.FALLING, states.RISING):
            time.sleep(options.fast)
        else:
            time.sleep(options.interval)

def main ():
    options = parse()
    setup_logging(options.debug, options.silent, options.name,
                  options.syslog_facility, not options.no_syslog)
    if options.pid:
        options.pid.write("{0}\n".format(os.getpid()))
        options.pid.close()
    try:
        # Setup IP to use
        options.ips = options.ips or loopback_ips(options.label)
        if not options.ips:
            raise RuntimeError("No IP found")
        if options.ip_setup:
            setup_ips(options.ips, options.label, options.sudo)
        options.ips = collections.deque(options.ips)
        options.ips.rotate(-options.start_ip)
        options.ips = list(options.ips)
        # Main loop
        loop(options)
    except Exception as e:
        logger.exception("Uncatched exception: %s", e)

if __name__ == "__main__":
    main()



# encoding: utf-8
"""
refresh.py

Created by Thomas Mangin on 2012-07-19.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from struct import unpack
from struct import error

from exabgp.protocol.family import AFI
from exabgp.protocol.family import SAFI
from exabgp.bgp.message import Message
from exabgp.bgp.message.notification import Notify

# =================================================================== Notification
# A Notification received from our peer.
# RFC 4271 Section 4.5


class Reserved (int):
	def __str__ (self):
		if self == 0:
			return 'query'
		if self == 1:
			return 'begin'
		if self == 2:
			return 'end'
		return 'invalid'


class RouteRefresh (Message):
	ID = Message.CODE.ROUTE_REFRESH
	TYPE = chr(Message.CODE.ROUTE_REFRESH)

	request = 0
	start = 1
	end = 2

	def __init__ (self, afi, safi, reserved=0):
		self.afi = AFI(afi)
		self.safi = SAFI(safi)
		self.reserved = Reserved(reserved)

	def message (self):
		return self._message('%s%s%s' % (self.afi.pack(),chr(self.reserved),self.safi.pack()))

	def __str__ (self):
		return "REFRESH"

	def extensive (self):
		return 'route refresh %s/%d/%s' % (self.afi,self.reserved,self.safi)

	def families (self):
		return self._families[:]

	@classmethod
	def unpack_message (cls, data, _):
		try:
			afi,reserved,safi = unpack('!HBB',data)
		except error:
			raise Notify(7,1,'invalid route-refresh message')
		if reserved not in (0,1,2):
			raise Notify(7,2,'invalid route-refresh message subtype')
		return RouteRefresh(afi,safi,reserved)

	def __eq__ (self, other):
		if not isinstance(other, RouteRefresh):
			return False
		if self.afi != other.afi:
			return False
		if self.safi != other.safi:
			return False
		if self.reserved != other.reserved:
			return False
		return True



# encoding: utf-8
"""
text.py

Created by Thomas Mangin on 2009-08-25.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from exabgp.configuration.ancient import Configuration
from exabgp.configuration.format import formated

from exabgp.protocol.family import AFI
from exabgp.protocol.family import SAFI
from exabgp.protocol.family import Family
from exabgp.protocol.ip import IP
from exabgp.bgp.message import OUT

from exabgp.bgp.message.update.nlri.prefix import Prefix
from exabgp.bgp.message.update.nlri.mpls import MPLS
from exabgp.bgp.message.refresh import RouteRefresh
from exabgp.bgp.message.operational import Advisory
from exabgp.bgp.message.operational import Query
from exabgp.bgp.message.operational import Response

from exabgp.rib.change import Change


# ========================================================================= Text
#

class Text (Configuration):
	def __init__ (self):
		Configuration.__init__(self,'','')
		# part of parent API, done here to remove pylint warning attribute-defined-outside-init
		self._nexthopself = None

	def parse_api_route (self, command, peers, action):
		tokens = formated(command).split(' ')[1:]
		number = len(tokens)

		if number < 1:
			return False

		message = tokens[0]

		if message not in ('route',):
			return False

		if number == 2 and action == 'withdraw' and 'next-hop' not in tokens:
			tokens.extend(['next-hop','0.0.0.0'])

		changes = []
		if 'self' in command:
			for peer,nexthop in peers.iteritems():
				scope = [{}]
				self._nexthopself = nexthop
				if not self._single_static_route(scope,tokens[1:]):
					self._nexthopself = None
					return False
				for change in scope[0]['announce']:
					changes.append((peer,change))
			self._nexthopself = None
		else:
			scope = [{}]
			if not self._single_static_route(scope,tokens[1:]):
				return False
			for peer in peers:
				for change in scope[0]['announce']:
					changes.append((peer,change))

		if action == 'withdraw':
			for (peer,change) in changes:
				change.nlri.action = OUT.WITHDRAW
		return changes

	def parse_api_vpls (self, command, peers, action):
		tokens = formated(command).split(' ')[1:]
		if len(tokens) < 4:
			return False
		if tokens[0] != 'vpls':
			return False
		changes = []
		if 'self' in command:
			for peer,nexthop in peers.iteritems():
				scope = [{}]
				self._nexthopself = nexthop
				if not self._single_l2vpn_vpls(scope,tokens[1:]):
					self._nexthopself = None
					return False
				for change in scope[0]['announce']:
					changes.append((peer,change))
			self._nexthopself = None
		else:
			scope = [{}]
			if not self._single_l2vpn_vpls(scope,tokens[1:]):
				return False
			for peer in peers:
				for change in scope[0]['announce']:
					changes.append((peer,change))
		if action == 'withdraw':
			for (peer,change) in changes:
				change.nlri.action = OUT.WITHDRAW
		return changes

	def parse_api_attribute (self, command, peers, action):
		# This is a quick solution which does not support next-hop self
		attribute,nlris = command.split('nlri')
		route = '%s route 0.0.0.0/0 %s' % (action, ' '.join(attribute.split()[2:]))
		parsed = self.parse_api_route(route,peers,action)
		if parsed in (True,False,None):
			return parsed
		attributes = parsed[0][1].attributes
		nexthop = parsed[0][1].nlri.nexthop
		changes = []
		for nlri in nlris.split():
			ip,mask = nlri.split('/')
			change = Change(
				MPLS(
					afi=IP.toafi(ip),
					safi=IP.tosafi(ip),
					packed=IP.pton(ip),
					mask=int(mask),
					nexthop=nexthop.packed,
					action=action
				),
				attributes
			)
			if action == 'withdraw':
				change.nlri.action = OUT.WITHDRAW
			else:
				change.nlri.action = OUT.ANNOUNCE
			changes.append((peers.keys(),change))
		return changes

	def parse_api_flow (self, command, action):
		command = command.replace('{','{\\n').replace('}','}\\n').replace(';',';\\n').replace('\\n\\n','\\n')
		self._tokens = self._tokenise(' '.join(formated(command).split(' ')[2:]).split('\\n'))
		scope = [{}]
		if not self._dispatch(scope,'flow',['route',],[],['root']):
			return False
		if not self._check_flow_route(scope):
			return False
		changes = scope[0]['announce']
		if action == 'withdraw':
			for change in changes:
				change.nlri.action = OUT.WITHDRAW
		return changes

	def parse_api_refresh (self, command):
		tokens = formated(command).split(' ')[2:]
		if len(tokens) != 2:
			return False
		afi = AFI.value(tokens.pop(0))
		safi = SAFI.value(tokens.pop(0))
		if afi is None or safi is None:
			return False
		return RouteRefresh(afi,safi)

	def parse_api_eor (self, command):
		tokens = formated(command).split(' ')[2:]
		number = len(tokens)

		if not number:
			return Family(1,1)

		if number != 2:
			return False

		afi = AFI.fromString(tokens[0])
		if afi == AFI.undefined:
			return False

		safi = SAFI.fromString(tokens[1])
		if safi == SAFI.undefined:
			return False

		return Family(afi,safi)

	def parse_api_operational (self, command):
		tokens = formated(command).split(' ',2)
		scope = [{}]

		if len(tokens) != 3:
			return False

		operational = tokens[0].lower()
		what = tokens[1].lower()

		if operational != 'operational':
			return False

		if what == 'asm':
			if not self._single_operational(Advisory.ASM,scope,['afi','safi','advisory'],tokens[2]):
				return False
		elif what == 'adm':
			if not self._single_operational(Advisory.ADM,scope,['afi','safi','advisory'],tokens[2]):
				return False
		elif what == 'rpcq':
			if not self._single_operational(Query.RPCQ,scope,['afi','safi','sequence'],tokens[2]):
				return False
		elif what == 'rpcp':
			if not self._single_operational(Response.RPCP,scope,['afi','safi','sequence','counter'],tokens[2]):
				return False
		elif what == 'apcq':
			if not self._single_operational(Query.APCQ,scope,['afi','safi','sequence'],tokens[2]):
				return False
		elif what == 'apcp':
			if not self._single_operational(Response.APCP,scope,['afi','safi','sequence','counter'],tokens[2]):
				return False
		elif what == 'lpcq':
			if not self._single_operational(Query.LPCQ,scope,['afi','safi','sequence'],tokens[2]):
				return False
		elif what == 'lpcp':
			if not self._single_operational(Response.LPCP,scope,['afi','safi','sequence','counter'],tokens[2]):
				return False
		else:
			return False

		operational = scope[0]['operational'][0]
		return operational



# encoding: utf-8
"""
capability/__init__.py

Created by Thomas Mangin on 2012-07-17.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from exabgp.bgp.message.open.capability.capability import Capability

from exabgp.bgp.message.open.capability.addpath import AddPath
from exabgp.bgp.message.open.capability.asn4 import ASN4
from exabgp.bgp.message.open.capability.graceful import Graceful
from exabgp.bgp.message.open.capability.mp import MultiProtocol
from exabgp.bgp.message.open.capability.ms import MultiSession
from exabgp.bgp.message.open.capability.operational import Operational
from exabgp.bgp.message.open.capability.refresh import RouteRefresh
from exabgp.bgp.message.open.capability.refresh import EnhancedRouteRefresh
from exabgp.bgp.message.open.capability.unknown import UnknownCapability

# Must be imported and registered for the register API to work
Capability.register_capability(AddPath)
Capability.register_capability(ASN4)
Capability.register_capability(Graceful)
Capability.register_capability(MultiProtocol)
Capability.register_capability(MultiSession,Capability.CODE.MULTISESSION_CISCO)
Capability.register_capability(MultiSession,Capability.CODE.MULTISESSION)
Capability.register_capability(Operational)
Capability.register_capability(RouteRefresh,Capability.CODE.ROUTE_REFRESH)
Capability.register_capability(RouteRefresh,Capability.CODE.ROUTE_REFRESH_CISCO)
Capability.register_capability(EnhancedRouteRefresh)
Capability.fallback_capability(UnknownCapability)
# End registration


class REFRESH (object):
	ABSENT   = 0x01
	NORMAL   = 0x02
	ENHANCED = 0x04



# encoding: utf-8
"""
process.py

Created by Thomas Mangin on 2014-06-22.
Copyright (c) 2014-2015 Exa Networks. All rights reserved.
"""

from exabgp.configuration.engine.location import Location
from exabgp.configuration.engine.raised import Raised
from exabgp.configuration.engine.section import Section
from exabgp.configuration.engine.parser import boolean

import os
import sys
import stat


# =============================================================== syntax_process

syntax_process = """\
process <name> {
	run </path/to/command with its args>  # the command can be quoted
	encoder text|json
	received {
		# "message" in notification,open,keepalive,update,refresh,operational,all
		<message> [
			parsed          # send parsed BGP data for "message"
			packets         # send raw BGP message for "message"
			consolidated    # group parsed and raw information in one JSON object
		]
		neighbor-changes  # state of peer change (up/down)
		parsed            # send parsed BGP data for all messages
		packets           # send raw BGP message for all messages
		consolidated      # group parsed and raw information for all messages
	}
	sent {
		packets           # send all generated BGP messages
	}
}
"""


# ================================================================ RaisedProcess

class RaisedProcess (Raised):
	syntax = syntax_process


# =============================================================== SectionProcess
#

class SectionProcess (Section):
	syntax = syntax_process
	name = 'process'

	def enter (self, tokeniser):
		Section.enter(self,tokeniser)
		self.content['encoder'] = 'text'

	def encoder (self, tokeniser):
		token = tokeniser()
		if token == '}':
			return
		if token not in ('text','json'):
			raise RaisedProcess(tokeniser,'invalid encoder')
		self.content['encoder'] = token

	def respawn (self, tokeniser):
		self.content['respawn'] = boolean(tokeniser,False)

	def run (self, tokeniser):
		command = tokeniser()

		prg,args = command.split(None,1)
		if prg[0] != '/':
			if prg.startswith('etc/exabgp'):
				parts = prg.split('/')
				path = [os.environ.get('ETC','etc'),] + parts[2:]
				prg = os.path.join(*path)
			else:
				prg = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]),prg))

		if not os.path.exists(prg):
			raise RaisedProcess(tokeniser,'can not locate the the program "%s"' % prg)

		# XXX: Yep, race conditions are possible, those are sanity checks not security ones ...
		s = os.stat(prg)

		if stat.S_ISDIR(s.st_mode):
			raise RaisedProcess(tokeniser,'can not execute directories "%s"' % prg)

		if s.st_mode & stat.S_ISUID:
			raise RaisedProcess(tokeniser,'refusing to run setuid programs "%s"' % prg)

		check = stat.S_IXOTH
		if s.st_uid == os.getuid():
			check |= stat.S_IXUSR
		if s.st_gid == os.getgid():
			check |= stat.S_IXGRP

		if not check & s.st_mode:
			raise RaisedProcess(tokeniser,'exabgp will not be able to run this program "%s"' % prg)

		self.content['run'] = '%s %s' % (prg,args)

	# all the action are the same

	def message (self, tokeniser):
		valid_messages = ['notification','open','keepalive','update','refresh','operational']
		valid_options = ['parsed','packets','consolidated']

		def printf (string):
			return (string[::-1].replace(',', ' and'[::-1], 1))[::-1]

		# location is set by our caller
		direction = self.location[-2]  # pylint: disable=E1101
		message = self.location[-1]    # pylint: disable=E1101
		actions = tokeniser()

		for (idx_line,idx_column,line,action) in actions:
			if action not in valid_options:
				raise RaisedProcess(Location(idx_line,idx_column,line),'invalid message option %s, valid options are "%s"' % (action,printf('", "'.join(valid_options))))

			messages = valid_messages if message == 'all' else [message]

			for m in messages:
				section = self.content.setdefault(direction,{}).setdefault(m,[])

				if action in section:
					raise RaisedProcess(Location(idx_line,idx_column,line),'duplicate action (%s) for message %s%s' % (
						action,
						m,
						" using the alis 'all'" if message == 'all' else ''
					))

				if 'consolidated' in section and len(section) > 0:
					raise RaisedProcess(Location(idx_line,idx_column,line),'consolidated can not be used with another keyword')

				section.append(action)

	def neighbor_changes (self, tokeniser):
		self.content.setdefault('received',{})['neighbor-changes'] = True

	# reveived global level

	def received_packets (self, tokeniser):
		for message in ['notification','open','keepalive','update','refresh','operational']:
			self.content.setdefault('received',{}).setdefault(message,[]).append('packets')

	def received_parsed (self, tokeniser):
		for message in ['notification','open','keepalive','update','refresh','operational']:
			self.content.setdefault('received',{}).setdefault(message,[]).append('parsed')

	def received_consolidated (self, tokeniser):
		for message in ['notification','open','keepalive','update','refresh','operational']:
			self.content.setdefault('received',{}).setdefault(message,[]).append('consolidated')

	# sent global level

	def sent_packets (self, tokeniser):
		for message in ['notification','open','keepalive','update','refresh','operational']:
			self.content.setdefault('sent',{}).setdefault(message,[]).append('packets')

	@classmethod
	def register (cls, registry, location):
		registry.register_class(cls)

		registry.register_hook(cls,'enter',location,'enter')

		registry.register_hook(cls,'action',location+['run'],'run')
		registry.register_hook(cls,'action',location+['encoder'],'encoder')
		registry.register_hook(cls,'action',location+['respawn'],'respawn')

		for received in (location+['received'],):

			registry.register_hook(cls,'enter',received,'enter_nameless')

			registry.register_hook(cls,'action',received+['neighbor-changes'],'neighbor_changes')

			for message in ['notification','open','keepalive','update','refresh','operational','all']:
				registry.register_hook(cls,'action',received+[message],'message')

			registry.register_hook(cls,'exit', received,'exit_nameless')

		registry.register_hook(cls,'enter',location+['sent'],'enter_nameless')
		registry.register_hook(cls,'action',location+['sent','packets'],'sent_packets')
		registry.register_hook(cls,'exit',location+['sent'],'exit_nameless')

		registry.register_hook(cls,'exit',location,'exit')



# encoding: utf-8
"""
__init__.py

Created by Thomas Mangin on 2010-01-15.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

import sys

from exabgp.application import run_exabgp
from exabgp.application import run_exabmp
from exabgp.application import run_cli
from exabgp.application import run_healthcheck

def main ():
	if len(sys.argv) == 1:
		run_exabgp()
		return

	if sys.argv[1] == 'bgp':
		sys.argv = sys.argv[1:]
		run_exabgp()
		return

	if sys.argv[1] == 'bmp':
		sys.argv = sys.argv[1:]
		run_exabgp()
		return

	if sys.argv[1] == 'healthcheck':
		sys.argv = sys.argv[1:]
		run_healthcheck()
		return

	if sys.argv[1] == 'cli':
		sys.argv = sys.argv[1:]
		run_cli()
		return

	run_exabgp()

if __name__ == '__main__':
	main()



# encoding: utf-8
"""
ms.py

Created by Thomas Mangin on 2012-07-17.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from exabgp.bgp.message.open.capability.capability import Capability

# ================================================================= MultiSession
#


class MultiSession (Capability,list):
	def __init__ (self):
		self.ID = Capability.CODE.MULTISESSION
		list.__init__(self)

	def set (self, data):
		self.extend(data)
		return self

	# XXX: FIXME: Looks like we could do with something in this Caoability
	def __str__ (self):
		info = ' (RFC)' if self.ID == Capability.CODE.MULTISESSION else ''
		return 'Multisession%s %s' % (info,' '.join([str(capa) for capa in self]))

	def json (self):
		variant = 'RFC' if self.ID == Capability.CODE.MULTISESSION else 'Cisco'
		return '{ "name": "multisession", "variant": "%s" ,"capabilities": [%s ] }' % (variant, ','.join(' %s' % str(capa) for capa in self))

	def extract (self):
		rs = [chr(0),]
		for v in self:
			rs.append(chr(v))
		return rs

	@staticmethod
	def unpack_capability (instance, data, capability=None):  # pylint: disable=W0613
		# XXX: FIXME: we should set that that instance was seen and raise if seen twice
		return instance



# encoding: utf-8
"""
originatorid.py

Created by Thomas Mangin on 2012-07-07.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from exabgp.protocol.ip import IPv4

from exabgp.bgp.message.update.attribute.attribute import Attribute


# ============================================================== OriginatorID (3)

class OriginatorID (Attribute,IPv4):
	ID = Attribute.CODE.ORIGINATOR_ID
	FLAG = Attribute.Flag.OPTIONAL
	CACHING = True

	__slots__ = []

	def pack (self, negotiated=None):
		return self._attribute(self.packed)

	@classmethod
	def unpack (cls, data, negotiated):
		return IPv4.unpack(data,cls)



# encoding: utf-8
"""
Enumeration.py

Created by Thomas Mangin on 2013-03-18.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""


# int are immutable once created: can not set ._str in __init__
class _integer (int):
	def __str__ (self):
		return self._str


class Enumeration (object):
	def __init__ (self, *names):
		for number, name in enumerate(names):
			# doing the .parent thing here instead
			number = _integer(pow(2,number))
			number._str = name
			setattr(self, name, number)


# Taken from Vincent Bernat
def Enum (*sequential):
	return type(str("Enum"), (), dict(zip(sequential, sequential)))



# encoding: utf-8
"""
update/__init__.py

Created by Thomas Mangin on 2010-01-15.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

# ================================================================== BGP States
#

class STATE (object):
	IDLE        = 0x01
	ACTIVE      = 0x02
	CONNECT     = 0x04
	OPENSENT    = 0x08
	OPENCONFIRM = 0x10
	ESTABLISHED = 0x20


# =================================================================== Direction
#

class OUT (object):
	ANNOUNCE = 0x01
	WITHDRAW = 0x02


class IN (object):
	ANNOUNCED = 0x01
	WITHDRAWN = 0x02


# ================================================================ Registration
#

from exabgp.bgp.message.message import Message
from exabgp.bgp.message.open import Open
from exabgp.bgp.message.update import Update
from exabgp.bgp.message.keepalive import KeepAlive
from exabgp.bgp.message.notification import Notification
from exabgp.bgp.message.refresh import RouteRefresh
from exabgp.bgp.message.operational import Operational

Message.register_message(Open)
Message.register_message(Update)
Message.register_message(KeepAlive)
Message.register_message(Notification)
Message.register_message(RouteRefresh)
Message.register_message(Operational)

from exabgp.bgp.message.notification import Notify

Message.klass_notify = Notify



# encoding: utf-8
"""
aigp.py

Created by Thomas Mangin on 2013-09-24.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from struct import pack
from struct import unpack

from exabgp.bgp.message.update.attribute.attribute import Attribute


# ========================================================================== TLV
#

# 0                   1                   2                   3
# 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
# |     Type      |         Length                |               |
# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+               |
# ~                                                               ~
# |                           Value                               |
# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+..........................

# Length: Two octets encoding the length in octets of the TLV,
# including the type and length fields.

class TLV (object):
	__slots__ = ['type','value']

	def __init__ (self, what, value):
		self.type = what
		self.value = value


class TLVS (list):
	__slots__ = []

	@staticmethod
	def unpack (data):
		def loop (data):
			while data:
				t = ord(data[0])
				l = unpack('!H',data[1:3])[0]
				v,data = data[3:l],data[l:]
				yield TLV(t,v)
		return TLVS(list(loop(data)))

	def pack (self):
		return ''.join('%s%s%s' % (chr(tlv.type),pack('!H',len(tlv.value)+3),tlv.value) for tlv in self)


# ==================================================================== AIGP (26)
#

class AIGP (Attribute):
	ID = Attribute.CODE.AIGP
	FLAG = Attribute.Flag.OPTIONAL
	CACHING = True
	TYPES = [1,]

	__slots__ = ['aigp','packed']

	def __init__ (self, aigp, packed=None):
		self.aigp = aigp
		if packed:
			self.packed = packed
		else:
			self.packed = self._attribute(aigp)

	def pack (self, negotiated):
		if negotiated.neighbor.aigp:
			return self.packed
		if negotiated.local_as == negotiated.peer_as:
			return self.packed
		return ''

	def __str__ (self):
		return '0x' + ''.join('%02x' % ord(_) for _ in self.aigp[-8:])

	@classmethod
	def unpack (cls, data, negotiated):
		if not negotiated.neighbor.aigp:
			# AIGP must only be accepted on configured sessions
			return None
		return cls(unpack('!Q',data[:8] & 0x000000FFFFFFFFFF),data[:8])



# encoding: utf-8
"""
community.py

Created by Thomas Mangin on 2009-11-05.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from exabgp.bgp.message.update.attribute.attribute import Attribute

from struct import pack

# ======================================================= ExtendedCommunity (16)
# XXX: Should subclasses register with transitivity ?


class ExtendedCommunity (Attribute):
	ID = Attribute.CODE.EXTENDED_COMMUNITY
	FLAG = Attribute.Flag.TRANSITIVE | Attribute.Flag.OPTIONAL

	registered_extended = {}

	@staticmethod
	def register_extended (klass):
		# COMMUNITY_TYPE and COMMUNITY_SUBTYPE are defined in subclasses
		ExtendedCommunity.registered_extended[(klass.COMMUNITY_TYPE & 0x0F,klass.COMMUNITY_SUBTYPE)] = klass

	# size of value for data (boolean: is extended)
	length_value = {False:7, True:6}
	name = {False: 'regular', True: 'extended'}

	__slots__ = ['community']

	def __init__ (self, community):
		# Two top bits are iana and transitive bits
		self.community = community

	def iana (self):
		return not not (self.community[0] & 0x80)

	def transitive (self):
		# bit set means "not transitive"
		#RFC4360:
		#        T - Transitive bit
		#
		#            Value 0: The community is transitive across ASes
		#
		#            Value 1: The community is non-transitive across ASes
		return not (self.community[0] & 0x40)

	def pack (self, negotiated=None):
		return self.community
	
	def _packedTypeSubtype(self, transitive=True):
		# if not transitive -> set the 'transitive' bit, as per RFC4360
		return pack(
			'!BB',
			self.COMMUNITY_TYPE if transitive else self.COMMUNITY_TYPE | 0x40,
			self.COMMUNITY_SUBTYPE
		)
	
	def json (self):
		h = 0x00
		for byte in self.community:
			h <<= 8
			h += ord(byte)
		return "%ld" % h

	def __str__ (self):
		h = 0x00
		for byte in self.community:
			h <<= 8
			h += ord(byte)
		return "0x%016X" % h

	def __len__ (self):
		return 8

	def __hash__ (self):
		return hash(self.community)

	def __cmp__ (self, other):
		if not isinstance(other, ExtendedCommunity):
			return -1
		return cmp(self.community,other.community)

	@staticmethod
	def unpack (data, negotiated=None):
		# 30/02/12 Quagga communities for soo and rt are not transitive when 4360 says they must be, hence the & 0x0FFF
		community = (ord(data[0]) & 0x0F,ord(data[1]))
		if community in ExtendedCommunity.registered_extended:
			return ExtendedCommunity.registered_extended[community].unpack(data)
		return ExtendedCommunity(data)



# encoding: utf-8
"""
link.py

Created by Thomas Mangin on 2015-03-31.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

import socket
from struct import calcsize
from collections import namedtuple

from exabgp.netlink.message import InfoMessage


# 0                   1                   2                   3
# 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
# |   Family    |   Reserved  |          Device Type              |
# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
# |                     Interface Index                           |
# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
# |                      Device Flags                             |
# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
# |                      Change Mask                              |
# +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

class Link (InfoMessage):
	class Header (object):
		PACK = 'BxHiII'
		LEN = calcsize(PACK)

	# linux/if_link.h
	format = namedtuple('Info', 'family type index flags change attributes')

	class Command (object):
		# linux/rtnetlink.h
		RTM_NEWLINK = 0x10  # Create a new network interface
		RTM_DELLINK = 0x11  # Destroy a network interface
		RTM_GETLINK = 0x12  # Retrieve information about a network interface (ifinfomsg)
		RTM_SETLINK = 0x13  # -

	class Type (object):
		class Family (object):
			AF_INET  = socket.AF_INET
			AF_INET6 = socket.AF_INET6

		class Device (object):
			IFF_UP            = 0x0001  # Interface is administratively up.
			IFF_BROADCAST     = 0x0002  # Valid broadcast address set.
			IFF_DEBUG         = 0x0004  # Internal debugging flag.
			IFF_LOOPBACK      = 0x0008  # Interface is a loopback interface.
			IFF_POINTOPOINT   = 0x0010  # Interface is a point-to-point link.
			IFF_NOTRAILERS    = 0x0020  # Avoid use of trailers.
			IFF_RUNNING       = 0x0040  # Interface is operationally up.
			IFF_NOARP         = 0x0080  # No ARP protocol needed for this interface.
			IFF_PROMISC       = 0x0100  # Interface is in promiscuous mode.
			IFF_ALLMULTI      = 0x0200  # Receive all multicast packets.
			IFF_MASTER        = 0x0400  # Master of a load balancing bundle.
			IFF_SLAVE         = 0x0800  # Slave of a load balancing bundle.
			IFF_MULTICAST     = 0x1000  # Supports multicast.

			IFF_PORTSEL       = 0x2000  # Is able to select media type via ifmap.
			IFF_AUTOMEDIA     = 0x4000  # Auto media selection active.
			IFF_DYNAMIC       = 0x8000  # Interface was dynamically created.

			IFF_LOWER_UP      = 0x10000  # driver signals L1 up
			IFF_DORMANT       = 0x20000  # driver signals dormant
			IFF_ECHO          = 0x40000  # echo sent packet

		class Attribute (object):
			IFLA_UNSPEC      = 0x00
			IFLA_ADDRESS     = 0x01
			IFLA_BROADCAST   = 0x02
			IFLA_IFNAME      = 0x03
			IFLA_MTU         = 0x04
			IFLA_LINK        = 0x05
			IFLA_QDISC       = 0x06
			IFLA_STATS       = 0x07

	@classmethod
	def getLinks (cls):
		return cls.extract(Link.Command.RTM_GETLINK)



# encoding: utf-8
"""
labels.py

Created by Thomas Mangin on 2012-07-08.
Copyright (c) 2014-2015 Orange. All rights reserved.
Copyright (c) 2014-2015 Exa Networks. All rights reserved.
"""

# TODO: take into account E-VPN specs that specify the role of the first bit of ESI
# (since draft-ietf-l2vpn-evpn-05)


# Ethernet Segment Identifier
class ESI (object):
	DEFAULT = ''.join(chr(0) for _ in range(0,10))
	MAX = ''.join(chr(0xFF) for _ in range(0,10))

	__slots__ = ['esi']

	def __init__ (self, esi=None):
		self.esi = self.DEFAULT if esi is None else esi

	def __str__ (self):
		if self.esi == self.DEFAULT:
			return "-"
		return ":".join('%02x' % ord(_) for _ in self.esi)

	def __repr__ (self):
		return self.__str__()

	def pack (self):
		return self.esi

	def __len__ (self):
		return 10

	def __cmp__ (self, other):
		if not isinstance(other,self.__class__):
			return -1
		if self.esi != other.esi:
			return -1
		return 0

	def __hash__ (self):
		return hash(self.esi)

	@classmethod
	def unpack (cls, data):
		return cls(data[:10])



# encoding: utf-8
"""
labels.py

Created by Thomas Mangin on 2012-07-08.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from struct import pack
from struct import unpack


# ======================================================================= Labels
# RFC 3107

class Labels (object):
	MAX = pow(2,20)-1

	__slots__ = ['labels','packed','_len']

	def __init__ (self, labels, bos=True):
		self.labels = labels
		packed = []
		for label in labels:
			# shift to 20 bits of the label to be at the top of three bytes and then truncate.
			packed.append(pack('!L',label << 4)[1:])
		# Mark the bottom of stack with the bit
		if packed and bos:
			packed.pop()
			packed.append(pack('!L',(label << 4) | 1)[1:])
		self.packed = ''.join(packed)
		self._len = len(self.packed)

	def pack (self):
		return self.packed

	def __len__ (self):
		return self._len

	def json (self):
		if self._len > 1:
			return '"label": [ %s ]' % ', '.join([str(_) for _ in self.labels])
		else:
			return ''

	def __str__ (self):
		if self._len > 1:
			return ' label [ %s ]' % ' '.join([str(_) for _ in self.labels])
		elif self._len == 1:
			return ' label %s' % self.labels[0]
		else:
			return ''

	@classmethod
	def unpack (cls, data):
		labels = []
		while True:
			label = unpack('!L',data[4:])
			labels.append(label >> 4)
			if label & 0x001:
				break
		return cls(labels,True)

Labels.NOLABEL = Labels([])



# encoding: utf-8
"""
location.py

Created by Thomas Mangin on 2014-06-22.
Copyright (c) 2014-2015 Exa Networks. All rights reserved.
"""

# ===================================================================== Location
# file location


class Location (object):
	def __init__ (self, idx_line=0, idx_column=0, line=''):
		self.line = line
		self.idx_line = idx_line
		self.idx_column = idx_column



# encoding: utf-8
"""
attribute.py

Created by Thomas Mangin on 2009-11-05.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from struct import pack

from exabgp.bgp.message.notification import Notify

from exabgp.util.cache import Cache


# ==================================================================== Attribute
#

class Attribute (object):
	# we need to define ID and FLAG inside of the subclasses
	ID   = 0x00
	FLAG = 0x00

	# Should this Attribute be cached
	CACHING = False

	# Registered subclasses we know how to decode
	registered_attributes = dict()

	# what this implementation knows as attributes
	attributes_known = []
	attributes_well_know = []
	attributes_optional = []

	# Are we caching Attributes (configuration)
	caching = False

	# The attribute cache per attribute ID
	cache = {}

	# ---------------------------------------------------------------------------

	# XXX: FIXME: The API of ID is a bit different (it can be instanciated)
	# XXX: FIXME: This is legacy. should we change to not be ?
	class CODE (int):
		__slots__ = []

		# This should move within the classes and not be here
		# RFC 4271
		ORIGIN             = 0x01
		AS_PATH            = 0x02
		NEXT_HOP           = 0x03
		MED                = 0x04
		LOCAL_PREF         = 0x05
		ATOMIC_AGGREGATE   = 0x06
		AGGREGATOR         = 0x07
		# RFC 1997
		COMMUNITY          = 0x08
		# RFC 4456
		ORIGINATOR_ID      = 0x09
		CLUSTER_LIST       = 0x0A  # 10
		# RFC 4760
		MP_REACH_NLRI      = 0x0E  # 14
		MP_UNREACH_NLRI    = 0x0F  # 15
		# RFC 4360
		EXTENDED_COMMUNITY = 0x10  # 16
		# RFC 4893
		AS4_PATH           = 0x11  # 17
		AS4_AGGREGATOR     = 0x12  # 18
		# RFC6514
		PMSI_TUNNEL        = 0x16  # 22
		# RFC5512
		TUNNEL_ENCAP       = 0x17  # 23
		AIGP               = 0x1A  # 26

		INTERNAL_NAME      = 0xFFFC
		INTERNAL_WITHDRAW  = 0xFFFD
		INTERNAL_WATCHDOG  = 0xFFFE
		INTERNAL_SPLIT     = 0xFFFF

		names = {
			ORIGIN:             'origin',
			AS_PATH:            'as-path',
			NEXT_HOP:           'next-hop',
			MED:                'med',              # multi-exit-disc
			LOCAL_PREF:         'local-preference',
			ATOMIC_AGGREGATE:   'atomic-aggregate',
			AGGREGATOR:         'aggregator',
			COMMUNITY:          'community',
			ORIGINATOR_ID:      'originator-id',
			CLUSTER_LIST:       'cluster-list',
			MP_REACH_NLRI:      'mp-reach-nlri',    # multi-protocol reacheable nlri
			MP_UNREACH_NLRI:    'mp-unreach-nlri',  # multi-protocol unreacheable nlri
			EXTENDED_COMMUNITY: 'extended-community',
			AS4_PATH:           'as4-path',
			AS4_AGGREGATOR:     'as4-aggregator',
			PMSI_TUNNEL:        'pmsi-tunnel',
			TUNNEL_ENCAP:       'tunnel-encaps',
			AIGP:               'aigp',
			0xfffc:             'internal-name',
			0xfffd:             'internal-withdraw',
			0xfffe:             'internal-watchdog',
			0xffff:             'internal-split',
		}

		def __str__ (self):
			return self.names.get(self,'unknown-attribute-%s' % hex(self))

		def __repr__ (self):
			return str(self)

		@classmethod
		def name (cls, self):
			return cls.names.get(self,'unknown-attribute-%s' % hex(self))

	# ---------------------------------------------------------------------------

	class Flag (int):
		EXTENDED_LENGTH = 0x10  # .  16 - 0001 0000
		PARTIAL         = 0x20  # .  32 - 0010 0000
		TRANSITIVE      = 0x40  # .  64 - 0100 0000
		OPTIONAL        = 0x80  # . 128 - 1000 0000

		MASK_EXTENDED   = 0xEF  # . 239 - 1110 1111
		MASK_PARTIAL    = 0xDF  # . 223 - 1101 1111
		MASK_TRANSITIVE = 0xBF  # . 191 - 1011 1111
		MASK_OPTIONAL   = 0x7F  # . 127 - 0111 1111

		__slots__ = []

		def __str__ (self):
			r = []
			v = int(self)
			if v & 0x10:
				r.append("EXTENDED_LENGTH")
				v -= 0x10
			if v & 0x20:
				r.append("PARTIAL")
				v -= 0x20
			if v & 0x40:
				r.append("TRANSITIVE")
				v -= 0x40
			if v & 0x80:
				r.append("OPTIONAL")
				v -= 0x80
			if v:
				r.append("UNKNOWN %s" % hex(v))
			return " ".join(r)

		def matches (self, value):
			return self | 0x10 == value | 0x10

	# ---------------------------------------------------------------------------

	def _attribute (self, value):
		flag = self.FLAG
		if flag & Attribute.Flag.OPTIONAL and not value:
			return ''
		length = len(value)
		if length > 0xFF:
			flag |= Attribute.Flag.EXTENDED_LENGTH
		if flag & Attribute.Flag.EXTENDED_LENGTH:
			len_value = pack('!H',length)
		else:
			len_value = chr(length)
		return "%s%s%s%s" % (chr(flag),chr(self.ID),len_value,value)

	def _len (self,value):
		length = len(value)
		return length + 3 if length <= 0xFF else length + 4

	def __eq__ (self, other):
		return self.ID == other.ID

	def __ne__ (self, other):
		return self.ID != other.ID

	@staticmethod
	def register_attribute (klass, attribute_id=None,flag=None):
		aid = klass.ID if attribute_id is None else attribute_id
		flg = klass.FLAG | Attribute.Flag.EXTENDED_LENGTH if flag is None else flag | Attribute.Flag.EXTENDED_LENGTH
		if (aid,flg) in klass.registered_attributes:
			raise RuntimeError('only one class can be registered per attribute')
		klass.registered_attributes[(aid,flg)] = klass
		klass.attributes_known.append(aid)
		if klass.FLAG & Attribute.Flag.OPTIONAL:
			Attribute.attributes_optional.append(aid)
		else:
			Attribute.attributes_well_know.append(aid)

	@classmethod
	def registered (cls, attribute_id, flag):
		return (attribute_id,flag | Attribute.Flag.EXTENDED_LENGTH) in cls.registered_attributes

	@classmethod
	def klass (cls, attribute_id, flag):
		key = (attribute_id,flag | Attribute.Flag.EXTENDED_LENGTH)
		if key in cls.registered_attributes:
			kls = cls.registered_attributes[key]
			kls.ID = attribute_id
			return kls
		# XXX: we do see some AS4_PATH with the partial instead of transitive bit set !!
		if attribute_id == Attribute.CODE.AS4_PATH:
			kls = cls.attributes_known[attribute_id]
			kls.ID = attribute_id
			return kls
		raise Notify (2,4,'can not handle attribute id %s' % attribute_id)

	@classmethod
	def unpack (cls, attribute_id, flag, data, negotiated):
		cache = cls.caching and cls.CACHING

		if cache and data in cls.cache.get(cls.ID,{}):
			return cls.cache[cls.ID].retrieve(data)

		key = (attribute_id,flag | Attribute.Flag.EXTENDED_LENGTH)
		if key in Attribute.registered_attributes.keys():
			instance = cls.klass(attribute_id,flag).unpack(data,negotiated)

			if cache:
				cls.cache[cls.ID].cache(data,instance)
			return instance

		raise Notify (2,4,'can not handle attribute id %s' % attribute_id)

	@classmethod
	def setCache (cls):
		if not cls.cache:
			for attribute in Attribute.CODE.names:
				if attribute not in cls.cache:
					cls.cache[attribute] = Cache()

Attribute.setCache()



"""
control.py

Created by Thomas Mangin on 2015-01-13.
Copyright (c) 2015-2015 Exa Networks. All rights reserved.
"""

import os
import sys
import select
import socket
import traceback


class Control (object):
	def __init__ (self, location=None,production=True):
		from exabgp.configuration.environment import environment
		self.location = location or environment.settings().api.socket

		self.unix = None
		self.sock = None
		self._production = production

	def __del__ (self):
		if self.sock:
			self.sock.close()
		if self.unix:
			self.unix.close()

		self.cleanup()

	# Can raise IOError
	def cleanup (self):
		if os.path.exists(self.location):
			os.remove(self.location)

	# Can raise IOError, socket.error
	def init (self):
		if not self.location:
			return False

		if not self.unix:
			try:
				# Can raise IOError
				self.cleanup()

				self.unix = socket.socket(socket.AF_UNIX,socket.SOCK_STREAM)
				# Can raise socket.error
				self.unix.bind(self.location)
				self.unix.listen(1)
				return True
			except IOError:
				print >> sys.stderr, 'could not access/delete socket file', self.location
			except socket.error:
				print >> sys.stderr, 'could not write socket file', self.location

			return False

	def write_standard (self, data):
		if not self._production:
			data = ''
		sys.stdout.write(data+'\n')
		sys.stdout.flush()

	def write_socket (self, data):
		self.sock.sendall(data)  # pylint: disable=E1101

	def read_socket (self, number):
		return self.sock.recv(number)  # pylint: disable=E1101

	def read_stdin (self, _):
		return sys.stdin.readline()

	def loop (self):
		report = ''

		while True:
			self.sock,_ = self.unix.accept()

			read = {
				sys.stdin: self.read_stdin,
				self.sock: self.read_socket,
			}

			write = {
				sys.stdin: self.write_socket,
				self.sock: self.write_standard,
			}

			store = {
				sys.stdin: '',
				self.sock: '',
			}

			looping = True
			while looping:
				ready,_,_ = select.select(read.keys(),[],[],1)
				if not ready:
					continue

				for fd in ready:
					r,_,_ = select.select([fd,],[],[],0)

					if r:
						new = read[fd](10240)
						if not new:
							looping = False
							break

						data = store[fd] + new

						while True:
							if '\n' not in data:
								store[fd] = data
								data = ''
								break
							report,data = data.split('\n',1)
							write[fd](report)

				if not self._production:
					return report

		# This is only for the unittesting code
		return report

	def run (self):
		try:
			if not self.init():
				return False
			return self.loop()
		except KeyboardInterrupt:
			self.cleanup()
		except Exception,exc:
			print >> sys.stderr, exc
			print >> sys.stderr,''
			traceback.print_exc(file=sys.stderr)
			sys.stderr.flush()
			self.cleanup()
			sys.exit(1)



# encoding: utf-8
"""
parser.py

Created by Thomas Mangin on 2014-07-01.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

# Generic ======================================================================
#

# As long as pack or unpack is needed here it means our API cleanup is not complete
from struct import pack
from struct import unpack

from socket import error as SocketError
from exabgp.protocol.ip import IP
from exabgp.bgp.message.open.asn import ASN
from exabgp.bgp.message.open.routerid import RouterID
from exabgp.bgp.message.open.holdtime import HoldTime


def string (tokeniser):
	return tokeniser()


def boolean (tokeniser, default):
	value = tokeniser()
	status = value.lower()
	if status in ('true','enable','enabled'):
		value = True
	elif status in ('false','disable','disabled'):
		value = False
	elif status in ('unset',):
		value = None
	else:
		tokeniser.rewind(value)
		return default
	return value


def md5 (tokeniser):
	value = tokeniser()
	if len(value) > 80:
		raise ValueError('MD5 password must be no larger than 80 characters')
	if not value:
		raise ValueError('value requires the value password as an argument (quoted or unquoted).  FreeBSD users should use "kernel" as the argument.')
	return value


def ttl (tokeniser):
	value = tokeniser()

	# XXX: FIXME: Should it be a subclass of int ?
	try:
		attl = int(value)
	except ValueError:
		if value in ('false','disable','disabled'):
			return None
		raise ValueError('invalid ttl-security "%s"' % value)

	if attl < 0:
		raise ValueError('ttl-security can not be negative')
	if attl >= 255:
		raise ValueError('ttl must be smaller than 256')
	return attl


def asn (tokeniser, value=None):
	value = tokeniser() if value is None else value
	try:
		if value.count('.'):
			high,low = value.split('.',1)
			as_number = (int(high) << 16) + int(low)
		else:
			as_number = int(value)
		return ASN(as_number)
	except ValueError:
		raise ValueError('"%s" is an invalid ASN' % value)


def ip (tokeniser):
	value = tokeniser()
	try:
		return IP.create(value)
	except (IndexError,ValueError,SocketError):
		raise ValueError('"%s" is an invalid IP address' % value)


def routerid (tokeniser):
	return RouterID(tokeniser())


def holdtime (tokeniser):
	value = tokeniser()
	try:
		hold_time = HoldTime(value)
	except ValueError:
		raise ValueError ('"%s" is an invalid hold-time' % value)

	if hold_time < 3 and hold_time != 0:
		raise ValueError('holdtime must be zero or at least three seconds')
	# XXX: FIXME: add HoldTime.MAX and reference it ( pow -1 )
	if hold_time >= pow(2,16):
		raise ValueError('holdtime must be smaller than %d' % pow(2,16))
	return hold_time


# Attributes ===================================================================
#


# ==================================================================== Attribute
#

from exabgp.bgp.message.update.attribute.attribute import Attribute


def attribute (tokeniser):
	start = tokeniser()
	if start != '[':
		raise ValueError('invalid attribute, does not starts with [')

	code = tokeniser().lower()
	if not code.startswith('0x'):
		raise ValueError('invalid attribute, code is not 0x hexadecimal')
	try:
		code = int(code[2:],16)
	except ValueError:
		raise ValueError('invalid attribute, code is not 0x hexadecimal')

	flag = tokeniser().lower()
	if not flag.startswith('0x'):
		raise ValueError('invalid attribute, flag is not 0x hexadecimal')
	try:
		flag = int(flag[2:],16)
	except ValueError:
		raise ValueError('invalid attribute, flag is not 0x hexadecimal')

	data = tokeniser().lower()
	if not data.startswith('0x'):
		raise ValueError('invalid attribute, data is not 0x hexadecimal')
	if not len(data) % 2:
		raise ValueError('invalid attribute, data is not 0x hexadecimal')
	data = ''.join(chr(int(data[_:_+2],16)) for _ in range(2,len(data),2))

	end = tokeniser()
	if end != ']':
		raise ValueError('invalid attribute, does not ends with ]')

	# XXX: FIXME: class Attribute should have an unpack function which does that
	from exabgp.bgp.message.update.attribute.generic import GenericAttribute

	for ((ID,flag),klass) in Attribute.registered_attributes.iteritems():
		if code == ID and flag == klass.FLAG:
			return klass(data)
	return GenericAttribute(code,flag,data)


# ====================================================================== NextHop
#

from exabgp.bgp.message.update.attribute.nexthop import NextHop


def next_hop (tokeniser):
	value = tokeniser()
	if value.lower() == 'self':
		# next-hop self is unsupported yet
		raise ValueError('unsupported yet on new format')
	else:
		return NextHop(value)


# ======================================================================= Origin
#

from exabgp.bgp.message.update.attribute.origin import Origin


def origin (tokeniser):
	value = tokeniser().lower()
	if value == 'igp':
		return Origin(Origin.IGP)
	if value == 'egp':
		return Origin(Origin.EGP)
	if value == 'incomplete':
		return Origin(Origin.INCOMPLETE)
	raise ValueError('unknown origin %s' % value)


# ========================================================================== MED
#

from exabgp.bgp.message.update.attribute.med import MED


def med (tokeniser):
	value = tokeniser()
	if not value.isdigit():
		raise ValueError('invalid MED %s' % value)
	return MED(value)


# ======================================================================= ASPath
#

from exabgp.bgp.message.update.attribute.aspath import ASPath


def aspath (tokeniser):
	as_seq = []
	as_set = []
	value = tokeniser()
	inset = False
	try:
		if value == '[':
			while True:
				value = tokeniser()
				if value == ',':
					continue
				if value in ('(','['):
					inset = True
					while True:
						value = tokeniser()
						if value == ')':
							break
						as_set.append(asn(tokeniser,value))
				if value == ')':
					inset = False
					continue
				if value == ']':
					if inset:
						inset = False
						continue
					break
				as_seq.append(ASN(tokeniser,value))
		else:
			as_seq.append(asn(tokeniser,value))
	except ValueError:
		raise ValueError('could not parse as-path')
	return ASPath(as_seq,as_set)


# ============================================================== LocalPreference
#

from exabgp.bgp.message.update.attribute.localpref import LocalPreference


def local_preference (tokeniser):
	value = tokeniser()
	if not value.isdigit():
		raise ValueError('invalid local preference %s' % value)
	return LocalPreference(value)


# ==================================================================== Community
#

from exabgp.bgp.message.update.attribute.atomicaggregate import AtomicAggregate


def atomic_aggregate (tokeniser):
	return AtomicAggregate()


# =================================================================== Aggregator
#

from exabgp.bgp.message.update.attribute.aggregator import Aggregator


def aggregator (tokeniser):
	value = tokeniser()
	if value != '(':
		tokeniser.rewind(value)
		return None

	try:
		as_number,address = tokeniser().split(':')
	except (ValueError,IndexError):
		raise ValueError('invalid aggregator')

	value = tokeniser()
	if value != ')':
		raise ValueError('invalid aggregator')

	local_as = ASN(as_number)
	local_address = RouterID(address)

	# XXX: This is buggy it can be an Aggregator4
	return Aggregator(local_as,local_address)


# ================================================================= OriginatorID
#

from exabgp.bgp.message.update.attribute.originatorid import OriginatorID


def originator_id (tokeniser):
	value = tokeniser()
	if not value.isdigit():
		raise ValueError('invalid Originator ID %s' % value)
	return OriginatorID(value)


# ================================================================== ClusterList
#

from exabgp.bgp.message.update.attribute.clusterlist import ClusterList
from exabgp.bgp.message.update.attribute.clusterlist import ClusterID


def cluster_list (tokeniser):
	clusterids = []
	value = tokeniser()
	try:
		if value == '[':
			while True:
				value = tokeniser()
				if value == ']':
					break
				clusterids.append(ClusterID(value))
		else:
			clusterids.append(ClusterID(value))
		if not clusterids:
			raise ValueError('no cluster-id in the cluster list')
		return ClusterList(clusterids)
	except ValueError:
		raise ValueError('invalud cluster list')


# ==================================================================== Community
#

from exabgp.bgp.message.update.attribute.community.community import Community


def _community (value):
	separator = value.find(':')
	if separator > 0:
		prefix = value[:separator]
		suffix = value[separator+1:]

		if not prefix.isdigit() or not suffix.isdigit():
			raise ValueError('invalid community %s' % value)

		prefix, suffix = int(prefix), int(suffix)

		# XXX: FIXME: add a Community.MAX to pow(2,16) -1
		if prefix >= pow(2,16):
			raise ValueError('invalid community %s (prefix too large)' % value)

		# XXX: FIXME: add a Community.MAX to pow(2,16) -1
		if suffix >= pow(2,16):
			raise ValueError('invalid community %s (suffix too large)' % value)

		return Community(pack('!L',(prefix << 16) + suffix))

	elif value[:2].lower() == '0x':
		value = long(value,16)
		# XXX: FIXME: add a Community.MAX to pow(2,16) -1
		if value >= pow(2,32):
			raise ValueError('invalid community %s (too large)' % value)
		return Community(pack('!L',value))
	else:
		low = value.lower()
		if low == 'no-export':
			return Community(Community.NO_EXPORT)
		elif low == 'no-advertise':
			return Community(Community.NO_ADVERTISE)
		elif low == 'no-export-subconfed':
			return Community(Community.NO_EXPORT_SUBCONFED)
		# no-peer is not a correct syntax but I am sure someone will make the mistake :)
		elif low == 'nopeer' or low == 'no-peer':
			return Community(Community.NO_PEER)
		elif low == 'blackhole':
			return Community(Community.BLACKHOLE)
		elif value.isdigit():
			value = unpack('!L',value)[0]
			if value >= pow(2,32):
				raise ValueError('invalid community %s (too large)' % value)
			return Community(pack('!L',value))
		else:
			raise ValueError('invalid community name %s' % value)


from exabgp.bgp.message.update.attribute.community.communities import Communities


def community (tokeniser):
	communities = Communities()

	value = tokeniser()
	if value == '[':
		while True:
			value = tokeniser()
			if value == ']':
				break
			communities.add(_community(value))
	else:
		communities.add(_community(value))

	return communities


# ========================================================== ExtendedCommunities
#

from exabgp.bgp.message.update.attribute.community.extended.community import ExtendedCommunity


def _extended_community (value):
	if value[:2].lower() == '0x':
		if not len(value) % 2:
			raise ValueError('invalid extended community %s' % value)
		try:
			raw = ''
			for i in range(2,len(value),2):
				raw += chr(int(value[i:i+2],16))
		except ValueError:
			raise ValueError('invalid extended community %s' % value)
		if len(raw) != 8:
			raise ValueError('invalid extended community %s' % value)
		return ExtendedCommunity.unpack(raw)
	elif value.count(':'):
		_known_community = {
			# header and subheader
			'target':  chr(0x00)+chr(0x02),
			'target4': chr(0x02)+chr(0x02),
			'origin':  chr(0x00)+chr(0x03),
			'origin4': chr(0x02)+chr(0x03),
			'l2info':  chr(0x80)+chr(0x0A),
		}

		_size_community = {
			'target':  2,
			'target4': 2,
			'origin':  2,
			'origin4': 2,
			'l2info':  4,
		}

		components = value.split(':')
		command = 'target' if len(components) == 2 else components.pop(0)

		if command not in _known_community:
			raise ValueError('invalid extended community %s (only origin,target or l2info are supported) ' % command)

		if len(components) != _size_community[command]:
			raise ValueError('invalid extended community %s, expecting %d fields ' % (command,len(components)))

		header = _known_community[command]

		if command == 'l2info':
			# encaps, control, mtu, site
			return ExtendedCommunity.unpack(header+pack('!BBHH',*[int(_) for _ in components]))

		if command in ('target','origin'):
			# global admin, local admin
			ga,la = components

			if '.' in ga or '.' in la:
				gc = ga.count('.')
				lc = la.count('.')
				if gc == 0 and lc == 3:
					# ASN first, IP second
					return ExtendedCommunity.unpack(header+pack('!HBBBB',int(ga),*[int(_) for _ in la.split('.')]))
				if gc == 3 and lc == 0:
					# IP first, ASN second
					return ExtendedCommunity.unpack(header+pack('!BBBBH',*[int(_) for _ in ga.split('.')]+[int(la)]))
			else:
				if command == 'target':
					if ga.upper().endswith('L'):
						return ExtendedCommunity.unpack(_known_community['target4']+pack('!LH',int(ga[:-1]),int(la)))
					else:
						return ExtendedCommunity.unpack(header+pack('!HI',int(ga),int(la)))
				if command == 'origin':
					if ga.upper().endswith('L'):
						return ExtendedCommunity.unpack(_known_community['origin4']+pack('!LH',int(ga),int(la)))
					else:
						return ExtendedCommunity.unpack(header+pack('!HI',int(ga),int(la)))

			if command == 'target4':
				return ExtendedCommunity.unpack(_known_community['target4']+pack('!LH',int(ga[:-1]),int(la)),None)

			if command == 'orgin4':
				return ExtendedCommunity.unpack(_known_community['origin4']+pack('!LH',int(ga[:-1]),int(la)),None)

		raise ValueError('invalid extended community %s' % command)
	else:
		raise ValueError('invalid extended community %s - lc+gc' % value)


# This is the same code as community with a different parser, should be factored

from exabgp.bgp.message.update.attribute.community.extended.communities import ExtendedCommunities


def extended_community (tokeniser):
	communities = ExtendedCommunities()

	value = tokeniser()
	if value == '[':
		while True:
			value = tokeniser()
			if value == ']':
				break
			communities.add(_extended_community(value))
	else:
		communities.add(_extended_community(value))

	return communities


# ===================================================== Fake Attribute: Watchdog
#

def watchdog (tokeniser):
	class Watchdog (str):
		ID = Attribute.CODE.INTERNAL_WATCHDOG

	command = tokeniser()
	if command.lower() in ['announce','withdraw']:
		raise ValueError('invalid watchdog name %s' % command)
	return Watchdog(command)


# ===================================================== Fake Attribute: Withdraw
#

def withdraw (tokeniser=None):
	class Withdrawn (object):
		ID = Attribute.CODE.INTERNAL_WITHDRAW
	return Withdrawn()


# Qualifiers ===================================================================
#


# =========================================================== RouteDistinguisher
#

from exabgp.bgp.message.update.nlri.qualifier.rd import RouteDistinguisher


def rd (tokeniser):
	try:
		value = tokeniser()

		separator = value.find(':')
		if separator > 0:
			prefix = value[:separator]
			suffix = int(value[separator+1:])

		# XXX: FIXME: we need much more checks here instead that the blank try/except...

		if '.' in prefix:
			data = [chr(0),chr(1)]
			data.extend([chr(int(_)) for _ in prefix.split('.')])
			data.extend([chr(suffix >> 8),chr(suffix & 0xFF)])
			distinguisher = ''.join(data)
		else:
			number = int(prefix)
			if number < pow(2,16) and suffix < pow(2,32):
				distinguisher = chr(0) + chr(0) + pack('!H',number) + pack('!L',suffix)
			elif number < pow(2,32) and suffix < pow(2,16):
				distinguisher = chr(0) + chr(2) + pack('!L',number) + pack('!H',suffix)
			else:
				raise ValueError('invalid route-distinguisher %s' % value)
	except ValueError:
		raise ValueError('invalid route-distinguisher %s' % value)

	return RouteDistinguisher(distinguisher)


# ============================================================== PathInformation
#

from exabgp.bgp.message.update.nlri.qualifier.path import PathInfo


def path_information (tokeniser):
	try:
		pi = tokeniser()
		if pi.isdigit():
			return PathInfo(integer=int(pi))
		return PathInfo(ip=pi)
	except ValueError:
		raise ValueError('invaluid path-information')


# ============================================================== PathInformation
#

from exabgp.bgp.message.update.nlri.qualifier.labels import Labels


def label (tokeniser):
	labels = []
	value = tokeniser()
	try:
		if value == '[':
			while True:
				value = tokeniser()
				if value == ']':
					break
				labels.append(int(value))
		else:
			labels.append(int(value))
	except ValueError:
		raise ValueError('invalid label %s' % value)

	return Labels(labels)



"""
evpn/__init__.py

Created by Thomas Mangin on 2014-06-27.
Copyright (c) 2014-2015 Orange. All rights reserved.
"""

from exabgp.bgp.message.update.nlri.evpn.nlri import EVPN

from exabgp.bgp.message.update.nlri.evpn.ethernetad import EthernetAD
from exabgp.bgp.message.update.nlri.evpn.mac import MAC
from exabgp.bgp.message.update.nlri.evpn.multicast import Multicast
from exabgp.bgp.message.update.nlri.evpn.segment import EthernetSegment

# EVPN.register_evpn(EthernetAD)
EVPN.register_evpn(MAC)
EVPN.register_evpn(Multicast)
# EVPN.register_evpn(EthernetSegment)



#!/usr/bin/env python
# encoding: utf-8
"""
peer.py

Created by Thomas Mangin on 2013-02-20.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

import os
import sys

import pwd
import socket
import select
import asyncore

from struct import unpack

from exabgp.reactor.network.error import error
from exabgp.reactor.api.encoding import JSON
from exabgp.bgp.message.update import Update

from exabgp.bmp.header import Header
from exabgp.bmp.message import Message
from exabgp.bmp.negotiated import FakeNegotiated


class BMPHandler (asyncore.dispatcher_with_send):
	wire = False
	update = True

	def announce (self, *args):
		print >> self.fd, self.ip, self.port, ' '.join(str(_) for _ in args) if len(args) > 1 else args[0]

	def setup (self, env, ip, port):
		self.handle = {
			Message.ROUTE_MONITORING:       self._route,
			Message.STATISTICS_REPORT:      self._statistics,
			Message.PEER_DOWN_NOTIFICATION: self._peer,
		}
		self.asn4 = env.bmp.asn4
		self.use_json = env.bmp.json
		self.fd = env.fd
		self.ip = ip
		self.port = port
		self.json = JSON('3.4.8')
		return self

	def _read_data (self, number):
		header = ''
		left = number
		while left:
			try:
				r,_,_ = select.select([self], [], [], 1.0)
			except select.error:
				return None

			if not r:
				continue

			try:
				data = self.recv(left)
			except socket.error,exc:
				if exc.args[0] in error.block:
					continue
				print "problem reading on socket", str(exc)
				return None

			left -= len(data)
			header += data

			if left and not data:
				# the TCP session is gone.
				print "TCP connection closed"
				self.close()
				return None
		return header

	def handle_read (self):
		data = self._read_data(44)
		if data is None:
			self.close()
			return
		header = Header(data)
		if not header.validate():
			print "closeing tcp connection following an invalid header"
			self.close()

		self.handle[header.message](header)

	def _route (self, header):
		bgp_header = self._read_data(19)
		if bgp_header is None:
			self.close()
			return
		length = unpack('!H',bgp_header[16:18])[0] - 19
		bgp_body = self._read_data(length)
		if bgp_body is None:
			self.close()
			return

		negotiated = FakeNegotiated(header,self.asn4)
		update = Update.unpack_message(bgp_body,negotiated)
		if self.use_json:
			print >> self.fd, self.json.bmp(self.ip,update)
		else:
			for route in update.routes:
				print >> self.fd, route.extensive()

	def _statistics (self, header):
		pass

	def _peer (self, header):
		pass


class BMPServer(asyncore.dispatcher):
	def __init__ (self, env):
		self.env = env
		host = env.bmp.host
		port = env.bmp.port
		asyncore.dispatcher.__init__(self)
		self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
		self.set_reuse_addr()
		self.bind((host, port))
		self.listen(5)

	def handle_accept (self):
		pair = self.accept()
		if pair is not None:
			# The if prevent invalid unpacking
			sock, addr = pair  # pylint: disable=W0633
			print "new BGP connection from", addr
			BMPHandler(sock).setup(self.env,*addr)


def drop ():
	uid = os.getuid()
	gid = os.getgid()

	if uid and gid:
		return

	for name in ['nobody',]:
		try:
			user = pwd.getpwnam(name)
			nuid = int(user.pw_uid)
			ngid = int(user.pw_uid)
		except KeyError:
			pass

	if not gid:
		os.setgid(ngid)
	if not uid:
		os.setuid(nuid)


from exabgp.configuration.environment import environment

_SPACE = {
	'space': ' '*33
}

HELP_STDOUT = """\
where logging should log
%(space)s syslog (or no setting) sends the data to the local syslog syslog
%(space)s host:<location> sends the data to a remote syslog server
%(space)s stdout sends the data to stdout
%(space)s stderr sends the data to stderr
%(space)s <filename> send the data to a file""" % _SPACE


environment.application = 'exabmp'
environment.configuration = {
	'pdb': {
		'enable': {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'false',
			'help':  'on program fault, start pdb the python interactive debugger',
		},
	},
	'bmp': {
		'host': {
			'read':  environment.nop,
			'write': environment.nop,
			'value': 'localhost',
			'help':  'port for the daemon to listen on',
		},
		'port': {
			'read':  environment.integer,
			'write': environment.nop,
			'value': '1790',
			'help': 'port for the daemon to listen on',
		},
		'asn4': {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'true',
			'help':  'are the route received by bmp in RFC4893 format',
		},
		'json': {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'true',
			'help':  'use json encoding of parsed route',
		},
	},
	'log': {
		'enable':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'true',
			'help':  'enable logging',
		},
		'level':  {
			'read':  environment.syslog_value,
			'write': environment.syslog_name,
			'value': 'INFO',
			'help':  'log message with at least the priority SYSLOG.<level>',
		},
		'destination':  {
			'read':  environment.unquote,
			'write': environment.quote,
			'value': 'stdout',
			'help':  HELP_STDOUT,
		},
		'all':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'false',
			'help':  'report debug information for everything',
		},
		'configuration':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'true',
			'help':  'report command parsing',
		},
		'reactor':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'true',
			'help':  'report signal received, command reload',
		},
		'daemon':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'true',
			'help':  'report pid change, forking, ...',
		},
		'processes':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'true',
			'help':  'report handling of forked processes',
		},
		'network':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'true',
			'help':  'report networking information (TCP/IP, network state,...)',
		},
		'packets':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'false',
			'help':  'report BGP packets sent and received',
		},
		'rib':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'false',
			'help':  'report change in locally configured routes',
		},
		'message':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'false',
			'help':  'report changes in route announcement on config reload',
		},
		'timers':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'false',
			'help':  'report keepalives timers',
		},
		'routes':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'false',
			'help':  'report received routes',
		},
		'parser':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'false',
			'help':  'report BGP message parsing details',
		},
		'short':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'false',
			'help':  'use short log format (not prepended with time,level,pid and source)',
		},
	},
	'cache': {
		'attributes':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'true',
			'help':  'cache all attributes (configuration and wire) for faster parsing',
		},
		'nexthops':  {
			'read':  environment.boolean,
			'write': environment.lower,
			'value': 'true',
			'help':  'cache routes next-hops (deprecated: next-hops are always cached)',
		},
	},
}

env = environment.setup('')


def main ():
	try:
		os.dup2(2,3)
		env.fd = os.fdopen(3, "w+")
	except Exception:
		print "can not setup a descriptor of FD 3 for route display"
		sys.exit(1)

	BMPServer(env)
	drop()

	try:
		asyncore.loop()
	except Exception:
		pass


if __name__ == '__main__':
	main()



# encoding: utf-8
"""
utils.py

Created by Thomas Mangin on 2009-09-06.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

# pylint: disable=too-few-public-methods,star-args,import-error

import os
import sys
import stat
import time
import syslog
import functools
import logging
import logging.handlers

from exabgp.util.od import od
from exabgp.util.hashtable import HashTable
from exabgp.configuration.environment import environment

_SHORT = {
	'CRITICAL': 'CRIT',
	'ERROR': 'ERR'
}


def short (name):
	return _SHORT.get(name.upper(),name.upper())


def _can_write (location):
	try:
		s  = os.stat(os.path.dirname(location))
	except OSError:
		return None
	mode = s[stat.ST_MODE]
	uid  = os.geteuid()
	gid  = os.getegid()

	return not not (
		((s[stat.ST_UID] == uid) and (mode & stat.S_IWUSR)) or
		((s[stat.ST_GID] == gid) and (mode & stat.S_IWGRP)) or
		(mode & stat.S_IWOTH)
	)


# This delays the evaluation of the od() function which is expensive
# hence why pylint too-few-public-method is disabled

class LazyFormat (object):
	def __init__ (self, prefix, message, formater=od):
		self.prefix = prefix
		self.message = message
		self.formater = formater

	def split (self, char):
		return str(self).split(char)

	def __str__ (self):
		formated = self.formater(self.message)
		return '%s (%d) %s' % (self.prefix,len(formated),formated)


class Logger (object):
	_instance = dict()
	_syslog = None

	_history = []
	_max_history = 20

	_config = ''
	_pid = os.getpid()
	_cwd = os.getcwd()

	def __new__ (cls):
		if cls._instance.get('class',None) is None:
			return super(Logger, cls).__new__(cls)
		else:
			return cls._instance['class']

	# we use os.pid everytime as we may fork and the class is instance before it

	def pdb (self, level):
		if self._option.pdb and level in ['CRITICAL','critical']:
			# not sure why, pylint reports an import error here
			import pdb
			pdb.set_trace()

	def config (self, config=None):
		if config is not None:
			self._config = config
		return self._config

	def history (self):
		return "\n".join(self._format(*_) for _ in self._history)

	def _record (self, timestamp, level, source, message):
		if len(self._history) > self._max_history:
			self._history.pop(0)
		self._history.append((timestamp,level,source,message))

	def _format (self, timestamp, level, source, message):
		if self.short:
			return message
		now = time.strftime('%a, %d %b %Y %H:%M:%S',timestamp)
		if self.destination in ['stderr','stdout']:
			return "%s | %-8s | %-6d | %-13s | %s" % (now,level,self._pid,source,message)
		if self.destination in ['', 'syslog'] or self.destination.startswith('host:'):
			return "%s[%d]: %-13s %s" % (environment.application,self._pid,source,message)
		return "%s: %-6d %-13s %s" % (environment.application,self._pid,source,message)

	def _prefixed (self, level, source, message):
		timestamp = time.localtime()
		self._record(timestamp,level,source,message)
		return self._format(timestamp,level,source,message)

	def __init__ (self):
		if self._instance.get('class',None) is not None:
			return

		self._instance['class'] = self

		command = environment.settings()
		self.short = command.log.short
		self.level = command.log.level

		self._option = HashTable()
		self._option.pdb           = command.debug.pdb
		self._option.reactor       = command.log.enable and (command.log.all or command.log.reactor)
		self._option.daemon        = command.log.enable and (command.log.all or command.log.daemon)
		self._option.processes     = command.log.enable and (command.log.all or command.log.processes)
		self._option.configuration = command.log.enable and (command.log.all or command.log.configuration)
		self._option.network       = command.log.enable and (command.log.all or command.log.network)
		self._option.wire          = command.log.enable and (command.log.all or command.log.packets)
		self._option.message       = command.log.enable and (command.log.all or command.log.message)
		self._option.rib           = command.log.enable and (command.log.all or command.log.rib)
		self._option.timer         = command.log.enable and (command.log.all or command.log.timers)
		self._option.routes        = command.log.enable and (command.log.all or command.log.routes)
		self._option.parser        = command.log.enable and (command.log.all or command.log.parser)

		if not command.log.enable:
			self.destination = ''
			return

		self.destination = command.log.destination

		self.restart(True)

	def _local_syslog (self):
		if sys.platform == "darwin":
			address = '/var/run/syslog'
		else:
			address = '/dev/log'
		if not os.path.exists(address):
			address = ('localhost', 514)
		handler = logging.handlers.SysLogHandler(address)

		self._syslog = logging.getLogger()
		self._syslog.setLevel(logging.DEBUG)
		self._syslog.addHandler(handler)
		return True

	def _remote_syslog (self, destination):
		# If the address is invalid, each syslog call will print an error.
		# See how it can be avoided, as the socket error is encapsulated and not returned
		address = (destination, 514)
		handler = logging.handlers.SysLogHandler(address)

		self._syslog = logging.getLogger()
		self._syslog.setLevel(logging.DEBUG)
		self._syslog.addHandler(handler)
		return True

	def _standard (self, facility):
		handler = logging.StreamHandler(getattr(sys,facility))

		self._syslog = logging.getLogger()
		self._syslog.setLevel(logging.DEBUG)
		self._syslog.addHandler(handler)
		return True

	def _file (self, destination):
		# folder
		logfile = os.path.realpath(os.path.normpath(os.path.join(self._cwd,destination)))
		can = _can_write(logfile)
		if can is True:
			handler = logging.handlers.RotatingFileHandler(logfile, maxBytes=5*1024*1024, backupCount=5)
		elif can is None:
			self.critical('ExaBGP can not access (perhaps as it does not exist) the log folder provided','logger')
			return False
		else:
			self.critical('ExaBGP does not have the right to write in the requested log directory','logger')
			return False

		self._syslog = logging.getLogger()
		self._syslog.setLevel(logging.DEBUG)
		self._syslog.addHandler(handler)
		return True

	def restart (self, first=False):
		if first:
			destination = 'stderr'
		else:
			if self._syslog:
				for handler in self._syslog.handlers:
					self._syslog.removeHandler(handler)
			destination = self.destination

		try:
			if destination in ('','syslog'):
				return self._local_syslog()

			if destination.startswith('host:'):
				return self._remote_syslog(destination[5:].strip())

			if destination in ('stdout','stderr'):
				return self._standard(destination)

			return self._file(destination)

		except IOError:
			self.critical('Can not set logging (are stdout/stderr closed?)','logger')
			return False

	def report (self, message, source='',level=''):
		for line in message.split('\n'):
			if self._syslog:
				self._syslog.debug(self._prefixed(level,source,line))
			else:
				print self._prefixed(level,source,line)
				sys.stdout.flush()

	debug = functools.partial(report,level='DEBUG')
	info = functools.partial(report,level='INFO')
	warning = functools.partial(report,level='WARNING')
	error = functools.partial(report,level='ERROR')
	critical = functools.partial(report,level='CRITICAL')

	def raw (self, message):
		for line in message.split('\n'):
			if self._syslog:
				self._syslog.critical(line)
			else:
				print line
				sys.stdout.flush()

	# show the message on the wire
	def network (self, message, recorder='info'):
		level = short(recorder)
		if self._option.network and getattr(syslog,'LOG_%s' % level) <= self.level:
			getattr(self,recorder.lower())(self,message,'network')
		else:
			self._record(time.localtime(),'network',recorder,message)
		self.pdb(recorder)

	# show the message on the wire
	def wire (self, message, recorder='debug'):
		level = short(recorder)
		if self._option.wire and getattr(syslog,'LOG_%s' % level) <= self.level:
			getattr(self,recorder.lower())(self,message,'wire')
		else:
			self._record(time.localtime(),'wire',recorder,message)
		self.pdb(recorder)

	# show the exchange of message between peers
	def message (self, message, recorder='info'):
		level = short(recorder)
		if self._option.message and getattr(syslog,'LOG_%s' % level) <= self.level:
			getattr(self,recorder.lower())(self,message,'message')
		else:
			self._record(time.localtime(),'message',recorder,message)
		self.pdb(recorder)

	# show the parsing of the configuration
	def configuration (self, message, recorder='info'):
		level = short(recorder)
		if self._option.configuration and getattr(syslog,'LOG_%s' % level) <= self.level:
			getattr(self,recorder.lower())(self,message,'configuration')
		else:
			self._record(time.localtime(),'configuration',recorder,message)
		self.pdb(recorder)

	# show the exchange of message generated by the reactor (^C and signal received)
	def reactor (self, message, recorder='info'):
		level = short(recorder)
		if self._option.reactor and getattr(syslog,'LOG_%s' % level) <= self.level:
			getattr(self,recorder.lower())(self,message,'reactor')
		else:
			self._record(time.localtime(),'reactor',recorder,message)
		self.pdb(recorder)

	# show the change of rib table
	def rib (self, message, recorder='info'):
		level = short(recorder)
		if self._option.rib and getattr(syslog,'LOG_%s' % level) <= self.level:
			getattr(self,recorder.lower())(self,message,'rib')
		else:
			self._record(time.localtime(),'rib',recorder,message)
		self.pdb(recorder)

	# show the change of rib table
	def timers (self, message, recorder='debug'):
		level = short(recorder)
		if self._option.timer and getattr(syslog,'LOG_%s' % level) <= self.level:
			getattr(self,recorder.lower())(self,message,'timers')
		else:
			self._record(time.localtime(),'timers',recorder,message)
		self.pdb(recorder)

	# show the exchange of message generated by the daemon feature (change pid, fork, ...)
	def daemon (self, message, recorder='info'):
		level = short(recorder)
		if self._option.daemon and getattr(syslog,'LOG_%s' % level) <= self.level:
			getattr(self,recorder.lower())(self,message,'daemon')
		else:
			self._record(time.localtime(),'daemon',recorder,message)
		self.pdb(recorder)

	# show the exchange of message generated by the forked processes
	def processes (self, message, recorder='info'):
		level = short(recorder)
		if self._option.processes and getattr(syslog,'LOG_%s' % level) <= self.level:
			getattr(self,recorder.lower())(self,message,'processes')
		else:
			self._record(time.localtime(),'processes',recorder,message)
		self.pdb(recorder)

	# show the exchange of message generated by the routes received
	def routes (self, message, recorder='info'):
		level = short(recorder)
		if self._option.routes and getattr(syslog,'LOG_%s' % level) <= self.level:
			getattr(self,recorder.lower())(self,message,'routes')
		else:
			self._record(time.localtime(),'routes',recorder,message)
		self.pdb(recorder)

	# show how the message received are parsed
	def parser (self, message, recorder='info'):
		level = short(recorder)
		if self._option.parser and getattr(syslog,'LOG_%s' % level) <= self.level:
			getattr(self,recorder.lower())(self,message,'parser')
		self.pdb(recorder)


class FakeLogger (object):
	def __getattr__ (self, name):
		def printf (data, _=None):
			sys.stdout.write('Fake logger [%s]\n' % str(data))
		return printf

# if __name__ == '__main__':
# 	logger = Logger()
# 	logger.wire('wire packet content')
# 	logger.message('message exchanged')
# 	logger.debug('debug test')



# encoding: utf-8
"""
vpls.py

Created by Nikita Shirokov on 2014-06-16.
Copyright (c) 2014-2015 Nikita Shirokov. All rights reserved.
Copyright (c) 2014-2015 Exa Networks. All rights reserved.
"""

from struct import unpack
from struct import pack
from exabgp.protocol.ip import IP
from exabgp.protocol.family import AFI
from exabgp.protocol.family import SAFI
from exabgp.bgp.message import OUT
from exabgp.bgp.message.notification import Notify
from exabgp.bgp.message.update.nlri.nlri import NLRI
from exabgp.bgp.message.update.nlri.qualifier.rd import RouteDistinguisher


def _unique ():
	value = 0
	while True:
		yield value
		value += 1

unique = _unique()


class VPLS (NLRI):

	__slots__ = ['action','nexthop','rd','base','offset','size','ve','unique']

	def __init__ (self, rd, ve, base, offset, size):
		NLRI.__init__(self,AFI.l2vpn,SAFI.vpls)
		self.action = OUT.ANNOUNCE
		self.nexthop = None
		self.rd = rd
		self.base = base
		self.offset = offset
		self.size = size
		self.ve = ve
		self.unique = unique.next()

	def __eq__ (self,other):
		return self.index() == other.index()

	def __ne__ (self, other):
		return not self.__eq__(other)

	def __hash__ (self):
		return hash(self.pack())

	def index (self):
		return self.pack()

	def pack (self, addpath=None):
		return '%s%s%s%s' % (
			'\x00\x11',  # pack('!H',17)
			self.rd.pack(),
			pack(
				'!HHH',
				self.ve,
				self.offset,
				self.size
			),
			pack(
				'!L',
				(self.base << 4) | 0x1
			)[1:]  # setting the bottom of stack, should we ?
		)

	# XXX: FIXME: we need an unique key here.
	# XXX: What can we use as unique key ?
	def json (self):
		content = ', '.join([
			self.rd.json(),
			'"endpoint": "%s"' % self.ve,
			'"base": "%s"' % self.base,
			'"offset": "%s"' % self.offset,
			'"size": "%s"' % self.size,
		])
		return '"vpls-%s": { %s }' % (self.unique, content)

	def extensive (self):
		return "vpls%s endpoint %s base %s offset %s size %s %s" % (
			self.rd,
			self.ve,
			self.base,
			self.offset,
			self.size,
			'' if self.nexthop is None else 'next-hop %s' % self.nexthop,
		)

	def __str__ (self):
		return self.extensive()

	@classmethod
	def unpack (cls, afi, safi, data, addpath, nexthop, action):
		# label is 20bits, stored using 3 bytes, 24 bits
		length, = unpack('!H',data[0:2])
		if len(data) != length+2:
			raise Notify(3,10,'l2vpn vpls message length is not consistent with encoded data')
		rd = RouteDistinguisher(data[2:10])
		ve,offset,size = unpack('!HHH',data[10:16])
		base = unpack('!L','\x00'+data[16:19])[0] >> 4
		nlri = cls(rd,ve,base,offset,size)
		nlri.action = action
		nlri.nexthop = IP.unpack(nexthop)
		return len(data), nlri



# encoding: utf-8
"""
community.py

Created by Thomas Mangin on 2009-11-05.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from struct import pack
from struct import unpack


# ==================================================================== Community
#

class Community (object):
	NO_EXPORT            = pack('!L',0xFFFFFF01)
	NO_ADVERTISE         = pack('!L',0xFFFFFF02)
	NO_EXPORT_SUBCONFED  = pack('!L',0xFFFFFF03)
	NO_PEER              = pack('!L',0xFFFFFF04)
	BLACKHOLE            = pack('!L',0xFFFF029A)

	cache = {}
	caching = True

	__slots__ = ['community','_str']

	def __init__ (self, community):
		self.community = community
		if community == self.NO_EXPORT:
			self._str = 'no-export'
		elif community == self.NO_ADVERTISE:
			self._str = 'no-advertise'
		elif community == self.NO_EXPORT_SUBCONFED:
			self._str = 'no-export-subconfed'
		elif community == self.NO_PEER:
			self._str = 'no-peer'
		elif community == self.BLACKHOLE:
			self._str = 'blackhole'
		else:
			self._str = "%d:%d" % unpack('!HH',self.community)

	def __cmp__ (self, other):
		if not isinstance(other,self.__class__):
			return -1
		if self.community != other.community:
			return -1
		return 0

	def json (self):
		return "[ %d, %d ]" % unpack('!HH',self.community)

	def pack (self, negotiated=None):
		return self.community

	def __str__ (self):
		return self._str

	def __len__ (self):
		return 4

	def __eq__ (self, other):
		return self.community == other.community

	def __ne__ (self, other):
		return self.community != other.community

	@classmethod
	def unpack (cls, community, negotiated):
		return cls(community)

	@classmethod
	def cached (cls, community):
		if cls.caching and community in cls.cache:
			return cls.cache[community]
		instance = cls(community)
		if cls.caching:
			cls.cache[community] = instance
		return instance

# Always cache well-known communities, they will be used a lot
if not Community.cache:
	Community.cache[Community.NO_EXPORT] = Community(Community.NO_EXPORT)
	Community.cache[Community.NO_ADVERTISE] = Community(Community.NO_ADVERTISE)
	Community.cache[Community.NO_EXPORT_SUBCONFED] = Community(Community.NO_EXPORT_SUBCONFED)
	Community.cache[Community.NO_PEER] = Community(Community.NO_PEER)
	Community.cache[Community.BLACKHOLE] = Community(Community.BLACKHOLE)



# encoding: utf-8
"""
operational/__init__.py

Created by Thomas Mangin on 2013-09-01.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from struct import pack
from struct import unpack

from exabgp.protocol.family import AFI
from exabgp.protocol.family import SAFI
from exabgp.bgp.message.open.routerid import RouterID
from exabgp.bgp.message import Message

# ========================================================================= Type
#

MAX_ADVISORY = 2048  # 2K


class Type (int):
	def pack (self):
		return pack('!H',self)

	def extract (self):
		return [pack('!H',self)]

	def __len__ (self):
		return 2

	def __str__ (self):
		pass


# ================================================================== Operational
#


class Operational (Message):
	ID = Message.CODE.OPERATIONAL
	TYPE = chr(Message.CODE.OPERATIONAL)

	registered_operational = dict()

	has_family = False
	has_routerid = False
	is_fault = False

	# really this should be called ID if not for the naming conflict
	class CODE (object):
		__slots__ = []

		NOP  = 0x00  # Not defined by the RFC
		# ADVISE
		ADM  = 0x01  # 01: Advisory Demand Message
		ASM  = 0x02  # 02: Advisory Static Message
		# STATE
		RPCQ = 0x03  # 03: Reachable Prefix Count Request
		RPCP = 0x04  # 04: Reachable Prefix Count Reply
		APCQ = 0x05  # 05: Adj-Rib-Out Prefix Count Request
		APCP = 0x06  # 06: Adj-Rib-Out Prefix Count Reply
		LPCQ = 0x07  # 07: BGP Loc-Rib Prefix Count Request
		LPCP = 0x08  # 08: BGP Loc-Rib Prefix Count Reply
		SSQ  = 0x09  # 09: Simple State Request
		# DUMP
		DUP  = 0x0A  # 10: Dropped Update Prefixes
		MUP  = 0x0B  # 11: Malformed Update Prefixes
		MUD  = 0x0C  # 12: Malformed Update Dump
		SSP  = 0x0D  # 13: Simple State Response
		# CONTROL
		MP   = 0xFFFE  # 65534: Max Permitted
		NS   = 0xFFFF  # 65535: Not Satisfied

	# XXX: FIXME: should be upper case
	name = ''
	category = ''
	code = CODE.NOP

	def __init__ (self, what):
		Message.__init__(self)
		self.what = Type(what)

	def _message (self, data):
		return Message._message(self,"%s%s%s" % (
			self.what.pack(),
			pack('!H',len(data)),
			data
		))

	def __str__ (self):
		return self.extensive()

	def extensive (self):
		return 'operational %s' % self.name

	@staticmethod
	def register_operational (klass):
		Operational.registered_operational[klass.code] = (klass.category,klass)

	@classmethod
	def unpack_message (cls, data, negotiated):  # pylint: disable=W0613
		what = Type(unpack('!H',data[0:2])[0])
		length = unpack('!H',data[2:4])[0]

		decode,klass = cls.registered_operational.get(what,('unknown',None))

		if decode == 'advisory':
			afi = unpack('!H',data[4:6])[0]
			safi = ord(data[6])
			data = data[7:length+4]
			return klass(afi,safi,data)
		elif decode == 'query':
			afi = unpack('!H',data[4:6])[0]
			safi = ord(data[6])
			routerid = RouterID.unpack(data[7:11])
			sequence = unpack('!L',data[11:15])[0]
			return klass(afi,safi,routerid,sequence)
		elif decode == 'counter':
			afi = unpack('!H',data[4:6])[0]
			safi = ord(data[6])
			routerid = RouterID.unpack(data[7:11])
			sequence = unpack('!L',data[11:15])[0]
			counter = unpack('!L',data[15:19])[0]
			return klass(afi,safi,routerid,sequence,counter)
		else:
			print 'ignoring ATM this kind of message'


# ============================================================ OperationalFamily
#

class OperationalFamily (Operational):
	has_family = True

	def __init__ (self, what, afi, safi, data=''):
		Operational.__init__(self,what)
		self.afi = AFI(afi)
		self.safi = SAFI(safi)
		self.data = data

	def family (self):
		return (self.afi,self.safi)

	def _message (self, data):
		return Operational._message(self,"%s%s%s" % (
			self.afi.pack(),
			self.safi.pack(),
			data
		))

	def message (self, negotiated):
		return self._message(self.data)


# =================================================== SequencedOperationalFamily
#


class SequencedOperationalFamily (OperationalFamily):
	__sequence_number = {}
	has_routerid = True

	def __init__ (self, what, afi, safi, routerid, sequence, data=''):
		OperationalFamily.__init__(self,what,afi,safi,data)
		self.routerid = routerid if routerid else None
		self.sequence = sequence if sequence else None
		self._sequence = self.sequence
		self._routerid = self.routerid

	def message (self, negotiated):
		self.sent_routerid = self.routerid if self.routerid else negotiated.sent_open.router_id
		if self.sequence is None:
			self.sent_sequence = (self.__sequence_number.setdefault(self.routerid,0) + 1) % 0xFFFFFFFF
			self.__sequence_number[self.sent_routerid] = self.sent_sequence
		else:
			self.sent_sequence = self.sequence

		return self._message("%s%s%s" % (
			self.sent_routerid.pack(),pack('!L',self.sent_sequence),
			self.data
		))


# =========================================================================== NS
#


class NS (object):
	MALFORMED   = 0x01  # Request TLV Malformed
	UNSUPPORTED = 0x02  # TLV Unsupported for this neighbor
	MAXIMUM     = 0x03  # Max query frequency exceeded
	PROHIBITED  = 0x04  # Administratively prohibited
	BUSY        = 0x05  # Busy
	NOTFOUND    = 0x06  # Not Found

	class _NS (OperationalFamily):
		is_fault = True

		def __init__ (self, afi, safi, sequence):
			OperationalFamily.__init__(
				self,
				Operational.CODE.NS,
				afi,safi,
				'%s%s' % (sequence,self.ERROR_SUBCODE)
			)

		def extensive (self):
			return 'operational NS %s %s/%s' % (self.name,self.afi,self.safi)

	class Malformed (_NS):
		name = 'NS malformed'
		ERROR_SUBCODE = '\x00\x01'  # pack('!H',MALFORMED)

	class Unsupported (_NS):
		name = 'NS unsupported'
		ERROR_SUBCODE = '\x00\x02'  # pack('!H',UNSUPPORTED)

	class Maximum (_NS):
		name = 'NS maximum'
		ERROR_SUBCODE = '\x00\x03'  # pack('!H',MAXIMUM)

	class Prohibited (_NS):
		name = 'NS prohibited'
		ERROR_SUBCODE = '\x00\x04'  # pack('!H',PROHIBITED)

	class Busy (_NS):
		name = 'NS busy'
		ERROR_SUBCODE = '\x00\x05'  # pack('!H',BUSY)

	class NotFound (_NS):
		name = 'NS notfound'
		ERROR_SUBCODE = '\x00\x06'  # pack('!H',NOTFOUND)


# ===================================================================== Advisory
#


class Advisory (object):
	class _Advisory (OperationalFamily):
		category = 'advisory'

		def extensive (self):
			return 'operational %s afi %s safi %s "%s"' % (self.name,self.afi,self.safi,self.data)

	class ADM (_Advisory):
		name = 'ADM'
		code = Operational.CODE.ADM

		def __init__ (self, afi, safi, advisory, routerid=None):
			utf8 = advisory.encode('utf-8')
			if len(utf8) > MAX_ADVISORY:
				utf8 = utf8[:MAX_ADVISORY-3] + '...'.encode('utf-8')
			Advisory._Advisory.__init__(
				self,Operational.CODE.ADM,
				afi,safi,
				utf8
			)

	class ASM (_Advisory):
		name = 'ASM'
		code = Operational.CODE.ASM

		def __init__ (self, afi, safi, advisory, routerid=None):
			utf8 = advisory.encode('utf-8')
			if len(utf8) > MAX_ADVISORY:
				utf8 = utf8[:MAX_ADVISORY-3] + '...'.encode('utf-8')
			Advisory._Advisory.__init__(
				self,Operational.CODE.ASM,
				afi,safi,
				utf8
			)


# ======================================================================== Query
#

class Query (object):
	class _Query (SequencedOperationalFamily):
		category = 'query'

		def __init__ (self, afi, safi, routerid, sequence):
			SequencedOperationalFamily.__init__(
				self,self.code,
				afi,safi,
				routerid,sequence
			)

		def extensive (self):
			if self._routerid and self._sequence:
				return 'operational %s afi %s safi %s router-id %s sequence %d' % (
					self.name,
					self.afi,self.safi,
					self._routerid,self._sequence,
				)
			return 'operational %s afi %s safi %s' % (self.name,self.afi,self.safi)

	class RPCQ (_Query):
		name = 'RPCQ'
		code = Operational.CODE.RPCQ

	class APCQ (_Query):
		name = 'APCQ'
		code = Operational.CODE.APCQ

	class LPCQ (_Query):
		name = 'LPCQ'
		code = Operational.CODE.LPCQ


# ===================================================================== Response
#

class Response (object):
	class _Counter (SequencedOperationalFamily):
		category = 'counter'

		def __init__ (self, afi, safi, routerid, sequence, counter):
			self.counter = counter
			SequencedOperationalFamily.__init__(
				self,self.code,
				afi,safi,
				routerid,sequence,
				pack('!L',counter)
			)

		def extensive (self):
			if self._routerid and self._sequence:
				return 'operational %s afi %s safi %s router-id %s sequence %d counter %d' % (
					self.name,
					self.afi,self.safi,
					self._routerid,self._sequence,
					self.counter
				)
			return 'operational %s afi %s safi %s counter %d' % (self.name,self.afi,self.safi,self.counter)

	class RPCP (_Counter):
		name = 'RPCP'
		code = Operational.CODE.RPCP

	class APCP (_Counter):
		name = 'APCP'
		code = Operational.CODE.APCP

	class LPCP (_Counter):
		name = 'LPCP'
		code = Operational.CODE.LPCP


# ========================================================================= Dump
#

class Dump (object):
	pass


Operational.register_operational(Advisory.ADM)
Operational.register_operational(Advisory.ASM)

Operational.register_operational(Query.RPCQ)
Operational.register_operational(Query.APCQ)
Operational.register_operational(Query.LPCQ)

Operational.register_operational(Response.RPCP)
Operational.register_operational(Response.APCP)
Operational.register_operational(Response.LPCP)



# encoding: utf-8
"""
store.py

Created by Thomas Mangin on 2009-11-05.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""

from collections import OrderedDict

from exabgp.protocol.family import AFI
from exabgp.protocol.family import SAFI

from exabgp.bgp.message import IN
from exabgp.bgp.message import OUT
from exabgp.bgp.message.update import Update
from exabgp.bgp.message.refresh import RouteRefresh
from exabgp.bgp.message.update.attribute.attributes import Attributes

# XXX: FIXME: we would not have to use so many setdefault if we pre-filled the dicts with the families


class Store (object):
	def __init__ (self, families):
		# XXX: FIXME: we can decide to not cache the routes we seen and let the backend do it for us and save the memory
		self._watchdog = {}
		self.cache = False
		self.families = families
		self.clear()

		# clear
		self._cache_attribute = {}
		self._seen = {}
		self._modify_nlri = {}
		self._modify_sorted = {}

		# clear + reset
		self._enhanced_refresh_start = []
		self._enhanced_refresh_delay = []

	# will resend all the routes once we reconnect
	def reset (self):
		# WARNING : this function can run while we are in the updates() loop too !
		self._enhanced_refresh_start = []
		self._enhanced_refresh_delay = []
		for _ in self.updates(True):
			pass

	# back to square one, all the routes are removed
	def clear (self):
		self._cache_attribute = {}
		self._seen = {}
		self._modify_nlri = {}
		self._modify_sorted = {}
		self.reset()

	def sent_changes (self, families=None):
		# families can be None or []
		requested_families = self.families if families is None else set(families).intersection(self.families)

		# we use list() to make a snapshot of the data at the time we run the command
		for family in requested_families:
			for change in self._seen.get(family,{}).values():
				if change.nlri.action == OUT.ANNOUNCE:
					yield change

	def resend (self, families, enhanced_refresh):
		# families can be None or []
		requested_families = self.families if not families else set(families).intersection(self.families)

		def _announced (family):
			for change in self._seen.get(family,{}).values():
				if change.nlri.action == OUT.ANNOUNCE:
					yield change
			self._seen[family] = {}

		if enhanced_refresh:
			for family in requested_families:
				if family not in self._enhanced_refresh_start:
					self._enhanced_refresh_start.append(family)
					for change in _announced(family):
						self.insert_announced(change,True)
		else:
			for family in requested_families:
				for change in _announced(family):
					self.insert_announced(change,True)

	# def dump (self):
	# 	# This function returns a hash and not a list as "in" tests are O(n) with lists and O(1) with hash
	# 	# and with ten thousands routes this makes an enormous difference (60 seconds to 2)
	# 	changes = {}
	# 	for family in self._seen.keys():
	# 		for change in self._seen[family].values():
	# 			if change.nlri.action == OUT.ANNOUNCE:
	# 				changes[change.index()] = change
	# 	return changes

	def queued_changes (self):
		for change in self._modify_nlri.values():
			yield change

	def replace (self, previous, changes):
		for change in previous:
			change.nlri.action = OUT.WITHDRAW
			self.insert_announced(change,True)

		for change in changes:
			self.insert_announced(change,True)

	def insert_announced_watchdog (self, change):
		watchdog = change.attributes.watchdog()
		withdraw = change.attributes.withdraw()
		if watchdog:
			if withdraw:
				self._watchdog.setdefault(watchdog,{}).setdefault('-',{})[change.index()] = change
				return True
			self._watchdog.setdefault(watchdog,{}).setdefault('+',{})[change.index()] = change
		self.insert_announced(change)
		return True

	def announce_watchdog (self, watchdog):
		if watchdog in self._watchdog:
			for change in self._watchdog[watchdog].get('-',{}).values():
				change.nlri.action = OUT.ANNOUNCE  # pylint: disable=E1101
				self.insert_announced(change)
				self._watchdog[watchdog].setdefault('+',{})[change.index()] = change
				self._watchdog[watchdog]['-'].pop(change.index())

	def withdraw_watchdog (self, watchdog):
		if watchdog in self._watchdog:
			for change in self._watchdog[watchdog].get('+',{}).values():
				change.nlri.action = OUT.WITHDRAW
				self.insert_announced(change)
				self._watchdog[watchdog].setdefault('-',{})[change.index()] = change
				self._watchdog[watchdog]['+'].pop(change.index())

	def insert_received (self, change):
		if not self.cache:
			return
		elif change.nlri.action == IN.ANNOUNCED:
			self._seen.get(change.nlri.family(),{})[change.index()] = change
		else:
			self._seen.get(change.nlri.family(),{}).pop(change.index(),None)

	def insert_announced (self, change, force=False):
		# WARNING: do not call change.nlri.index as it does not prepend the family
		# WARNING : this function can run while we are in the updates() loop

		# self._seen[family][nlri-index] = change

		# self._modify_nlri[nlri-index] = change : we are modifying this nlri
		# self._modify_sorted[attr-index][nlri-index] = change : add or remove the nlri
		# self._cache_attribute[attr-index] = attributes of one of the changes
		# and it allow to overwrite change easily :-)

		# import traceback
		# traceback.print_stack()
		# print "inserting", change.extensive()

		if not force and self._enhanced_refresh_start:
			self._enhanced_refresh_delay.append(change)
			return

		change_nlri_index = change.index()
		change_attr_index = change.attributes.index()

		dict_sorted = self._modify_sorted
		dict_nlri = self._modify_nlri
		dict_attr = self._cache_attribute

		# removing a route before we had time to announce it ?
		if change_nlri_index in dict_nlri:
			# pop removes the entry
			old_change = dict_nlri.pop(change_nlri_index)
			old_attr_index = old_change.attributes.index()
			# do not delete dict_attr, other routes may use it
			del dict_sorted[old_attr_index][change_nlri_index]
			if not dict_sorted[old_attr_index]:
				del dict_sorted[old_attr_index]

			# if we cache sent NLRI and this NLRI was never sent before, we do not need to send a withdrawal
			# as the route removed before we could announce it
			if self.cache and old_change.nlri.action == OUT.ANNOUNCE and change.nlri.action == OUT.WITHDRAW:
				if change_nlri_index not in self._seen.get(change.nlri.family(),{}):
					return

		if self.cache and not force:
			if change.nlri.action == OUT.ANNOUNCE:
				announced = self._seen.get(change.nlri.family(),{})
				if change_nlri_index in announced:
					old_change = announced[change_nlri_index]
					# it is a duplicate route
					if old_change.attributes.index() == change.attributes.index() and old_change.nlri.nexthop.index() == change.nlri.nexthop.index():
						return

		# add the route to the list to be announced
		dict_sorted.setdefault(change_attr_index,OrderedDict())[change_nlri_index] = change
		dict_nlri[change_nlri_index] = change

		if change_attr_index not in dict_attr:
			dict_attr[change_attr_index] = change.attributes

	def updates (self, grouped):
		dict_nlri = self._modify_nlri

		rr_announced = []

		for afi,safi in self._enhanced_refresh_start:
			rr_announced.append((afi,safi))
			yield Update(RouteRefresh(afi,safi,RouteRefresh.start),Attributes())

		dict_sorted = self._modify_sorted
		# Create a new dictionary as the outstanding routes are in dict_sorted.
		self._modify_sorted = {}
		dict_attr = self._cache_attribute

		for attr_index,dict_change in dict_sorted.items():
			if not dict_change:
				continue

			updates = {}
			changed = dict_change.values()
			attributes = dict_attr[attr_index]

			if not changed:
				continue

			for change in changed:
				updates.setdefault(change.nlri.family(),[]).append(change.nlri)

			family = change.nlri.family()

			# only yield once we have a consistent state, otherwise it will go wrong
			# as we will try to modify things we are iterating over and using

			if grouped and family == (AFI.ipv4,SAFI.unicast):
				for nlris in updates.itervalues():
					yield Update(nlris, attributes)
			else:
				for nlris in updates.itervalues():
					for nlri in nlris:
						yield Update([nlri,], attributes)

			if self.cache:
				announced = self._seen
				for change in changed:
					if change.nlri.action == OUT.ANNOUNCE:
						announced.setdefault(change.nlri.family(),{})[change.index()] = change
					else:
						family = change.nlri.family()
						if family in announced:
							announced[family].pop(change.index(),None)

		self._modify_nlri = {}

		if rr_announced:
			for afi,safi in rr_announced:
				self._enhanced_refresh_start.remove((afi,safi))
				yield Update(RouteRefresh(afi,safi,RouteRefresh.end),Attributes())

			for change in self._enhanced_refresh_delay:
				self.insert_announced(change,True)
			self._enhanced_refresh_delay = []

			for update in self.updates(grouped):
				yield update



# encoding: utf-8
"""
sequence.py

Created by Thomas Mangin on 2015-03-31.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""


class Sequence (int):
	_instance = dict()

	def __new__ (cls):
		cls._instance['next'] = cls._instance.get('next', 0) + 1
		return cls._instance['next']



# encoding: utf-8
"""
message.py

Created by Thomas Mangin on 2013-02-26.
Copyright (c) 2009-2015 Exa Networks. All rights reserved.
"""


class FakeAddPath (object):
	def send (self, afi, safi):  # pylint: disable=W0613
		return False

	def receive (self, afi, safi):  # pylint: disable=W0613
		return False


class FakeNegotiated (object):
	def __init__ (self, header, asn4):  # pylint: disable=W0613
		self.asn4 = asn4
		self.addpath = FakeAddPath()
